jQuery( function( $ ) {

	// Init after DOM is ready
	$( document ).ready( function() {
		init();
	} );

	// Functions to initiate
	function init() {
		wpoWcpbnAppendNewBatchNumberRow( '.wpo-wcpbn-table-new-row-simple' );
		wpoWcpbnAppendNewBatchMoreOfLast( '.wpo-wcpbn-table-more-of-last-simple' );

		// On variation opening
		$( document ).on( 'woocommerce_variations_loaded', function( event ) {
			wpoWcpbnAppendNewBatchNumberRow( '.wpo-wcpbn-table-new-row-variation' );
			wpoWcpbnAppendNewBatchMoreOfLast( '.wpo-wcpbn-table-more-of-last-variation' );
			wpoWcpbnDeleteBatchNumberRow( '.wpo-wcpbn-table-delete-db-button-variation' );
			wpoWcpbnSupplierAutocomplete();
			wpoWcpbnBatchNumberStatusColor();
			wpoWcpbnSyncProductWcStock();
		} );

		$( document ).on( 'woocommerce_load_order_items', function( event ) {
			wpoWcpbnEditOrderItem();
		} );

		wpoWcpbnDeleteBatchNumberRow( '.wpo-wcpbn-table-delete-db-button-simple' );
		wpoWcpbnDeleteBatchNumberRow( '#wpo-wcpbn-overview .delete-batch' );
		wpoWcpbnEditBatchNumberRow();
		wpoWcpbnBatchNumberStatusColor();
		wpoWcpbnSaveData( '.wpo-wcpbn-table-save-data' );
		wpoWcpbnEditOrderItem();
		wpoWcpbnSupplierAutocomplete();
		wpoWcpbnBatchNumberCheck();
		wpoWcpbnSyncProductWcStock();
		wpoWcpbnDisplaySettingsConditionally();
		wpoWcpbnToggleExportDateRange();
	}

	// Product Append New Batch Number Row
	function wpoWcpbnAppendNewBatchNumberRow( buttonClass ) {
		$( document ).off( 'click', buttonClass ).on( 'click', buttonClass, function( e ) { // fix to attach only one event at a time
			newBatchRowHtml( buttonClass, $( this ) ); // Append new row
		} );
	}

	// Product Append more of the last
	function wpoWcpbnAppendNewBatchMoreOfLast( buttonClass ) {
		$( document ).off( 'click', buttonClass ).on( 'click', buttonClass, function( e ) {
			newBatchRowHtml( buttonClass, $( this ), true ); // Append new row
		} );
	}

	function newBatchRowHtml( buttonClass, $elem, moreOfLast = false ) {
		$.ajax( {
			url:      wpo_wcpbn.ajaxurl,
			data: {
				action: 'new_batch_number_row_html'
			},
			type:     'POST',
			cache:    false,
			dataType: 'html',
			success: function( html ) {
				// if variations
				if ( $.inArray( buttonClass, [ '.wpo-wcpbn-table-new-row-variation', '.wpo-wcpbn-table-more-of-last-variation' ] ) != -1 ) {
					// get the variation id
					let variationId = $elem.closest( '.woocommerce_variation' ).find( ':input.variable_post_id' ).val();
					// add the id to the hidden input
					$elem.closest( '.wpo-wcpbn-table' ).find( 'tbody' ).append( html ).find( '.wpo-wcpbn-variation-id' ).val( variationId );
					$elem.closest( '.wpo-wcpbn-table' ).find( 'tbody' ).find( '.wpo-wcpbn-batch-number-id' ).attr( 'name', 'wpo-wcpbn-batch-number-id['+variationId+'][]' );

					if ( moreOfLast ) {
						// grab the newly created row
						let $newRow = $elem.closest( '.wpo-wcpbn-table' ).find( 'tbody > tr:last' );
						// get the last added batch
						ajax_get_last_added_batch( variationId, $newRow );
					}

				// if simple product
				} else if ( $.inArray( buttonClass, [ '.wpo-wcpbn-table-new-row-simple', '.wpo-wcpbn-table-more-of-last-simple' ] ) != -1 ) {
					// grab the post ID from the URL string
					let id = $elem.closest( '.wpo-wcpbn-table' ).data( 'product_id' );
					// add the id to the hidden input
					$elem.closest( '.wpo-wcpbn-table' ).find( 'tbody' ).append( html );
					$elem.closest( '.wpo-wcpbn-table' ).find( 'tbody' ).find( '.wpo-wcpbn-batch-number-id' ).attr( 'name', 'wpo-wcpbn-batch-number-id['+id+'][]' );

					if ( moreOfLast ) {
						// grab the newly created row
						let $newRow = $elem.closest( '.wpo-wcpbn-table' ).find( 'tbody > tr:last' );
						// get the last added batch
						ajax_get_last_added_batch( id, $newRow );
					}
				}
				// Fire autocomplete
				wpoWcpbnSupplierAutocomplete();
				// Fire batch number check
				wpoWcpbnBatchNumberCheck();
				// Fire delete event
				wpoWcpbnDeleteNewRow();
				// remove empty warning
				if ( $elem.closest( '.wpo-wcpbn-table-empty-warning' ).length ) {
					$elem.closest( '.wpo-wcpbn-table-empty-warning' ).remove();
				}
			}
		} );
	}

	function ajax_get_last_added_batch( id, $newRow ) {
		$.ajax( {
			url:      wpo_wcpbn.ajaxurl,
			data: {
				action: 'get_last_added_batch',
				id:	    id,
			},
			type:     'POST',
			cache:    false,
			dataType: 'html',
			success: function( response ) {
				response = $.parseJSON( response );
				if ( response.success ) {
					let lastBatch = response.data.last_batch;
					// loop throw new row 'tds' to get each input
					$newRow.find( 'td' ).each( function( i, v ) {
						let $row      = $( v );
						let rowType   = $row.attr( 'class' ).replace( 'column-', '' );
						let $rowInput = $row.find( 'input:visible' );

						switch( rowType ) {
							case 'expirydate':
								if ( lastBatch.date_expiry != '0000-00-00 00:00:00' || lastBatch.date_expiry != '1000-01-01 00:00:00' ) {
									let date = lastBatch.date_expiry.split( ' ' )[0]; // remove time
									$rowInput.val( date );
								}
								break;
							case 'batchnumber':
								$rowInput.val( lastBatch.batch_number );
								break;
							case 'supplier':
								$rowInput.val( lastBatch.supplier );
								break;
							case 'location':
								$rowInput.val( lastBatch.location );
								break;
							case 'quantities':
								$rowInput.val( lastBatch.quantity );
								break;
						}
					} );
				}
			}
		} );
	}

	// Product Delete Batch Number Row
	function wpoWcpbnDeleteBatchNumberRow( buttonClass ) {
		$( document ).on( 'click', buttonClass, function( e ) {
			let $elem = $( this );
			let id    = $elem.attr( 'data-id' );
			let pid   = $elem.attr( 'data-pid' );

			// show confirm alert
			if ( confirm( 'Are you sure? This is irreversible.' ) ) {
				
				// block UI
				$( '#woocommerce-product-data' ).block( {
					message: null,
					overlayCSS: {
						background: '#fff',
						opacity: 0.6,
					}
				} );
				
				// ajax request
				$.ajax( {
					url:   wpo_wcpbn.ajaxurl,
					data: {
						action:     'delete_batch_number_row',
						batch_id:   id,
						product_id: pid,
					},
					type:  'POST',
					cache: false,
					success: function( data ) {
						if ( buttonClass == '.wpo-wcpbn-table-delete-db-button-variation' || buttonClass == '#wpo-wcpbn-overview .delete-batch' ) {
							location.reload();
						} else {
							// reload table with the new data
							$elem.closest( '.wpo-wcpbn-group' ).load( document.URL + ' .wpo-wcpbn-wrapper' );
						}
						// unblock UI
						$( '#woocommerce-product-data' ).unblock();
					},
				} );
			}
		} );
	}

	// Product Edit Batch Number Row
	function wpoWcpbnEditBatchNumberRow() {
		$( document ).on( 'click', '.wpo-wcpbn-table-edit-row', function() {
			if ( $( this ).closest( 'tr' ).find( '.wpo-wcpbn-edit-quantity' ).is( ':hidden' ) ) {
				$( this ).closest( 'tr' ).find( 'input' ).prop( 'disabled', false );
				$( this ).closest( 'tr' ).find( 'select' ).prop( 'disabled', false );
				$( this ).closest( 'tr' ).find( '.wpo-wcpbn-initial-quantity' ).prop( 'disabled', true );
				$( this ).closest( 'tr' ).find( '.wpo-wcpbn-show-quantities' ).prop( 'disabled', true ).hide();
				$( this ).closest( 'tr' ).find( '.wpo-wcpbn-edit-quantity' ).show();
				$( this ).closest( 'tr' ).find( '.wpo-wcpbn-table-delete-row' ).show().css( 'display', 'inline' ).css( 'cursor', 'pointer' );
			} else if ( $( this ).closest( 'tr' ).find( '.wpo-wcpbn-edit-quantity' ).is( ':visible' ) ) {
				$( this ).closest( 'tr' ).find( 'input' ).prop( 'disabled', true );
				$( this ).closest( 'tr' ).find( 'select' ).prop( 'disabled', true );
				$( this ).closest( 'tr' ).find( '.wpo-wcpbn-initial-quantity' ).prop( 'disabled', false );
				$( this ).closest( 'tr' ).find( '.wpo-wcpbn-show-quantities' ).prop( 'disabled', true ).show();
				$( this ).closest( 'tr' ).find( '.wpo-wcpbn-edit-quantity' ).hide();
				$( this ).closest( 'tr' ).find( '.wpo-wcpbn-table-delete-row' ).hide();
			}
		} );
	}

	function wpoWcpbnBatchNumberStatusColor() {
		$( document ).on( 'change', '.wpo-wcpbn-group table td.column-status select', function( e ) {
			e.preventDefault();
			let values = $.map( $( this ).find( 'option' ) ,function( option ) {
				return option.value;
			});
			$( this ).removeClass( values.join( ' ' ) ).addClass( $( this ).val() );
		} ).trigger( 'change' );
	}

	function wpoWcpbnDeleteNewRow() {
		$( '.wpo-wcpbn-table-delete-new-row' ).on( 'click', function() {
			$( this ).parent().parent().remove();
		} );
	}

	function wpoWcpbnSaveData( buttonClass ) {
		$( document ).on( 'click', buttonClass, function( e ) {
			let $elem = $( this );
			let id    = $elem.closest( '.wpo-wcpbn-table' ).data( 'product_id' );

			// block UI
			$( '#woocommerce-product-data' ).block( {
				message: null,
				overlayCSS: {
					background: '#fff',
					opacity: 0.6
				}
			} );

			// form to save
			let action   = 'crud_batch_numbers_product';
			let formData = $( '#post' ).serialize() + '&action=' + action + '&post_id=' + id;

			// ajax request
			$.ajax( {
				url:   wpo_wcpbn.ajaxurl,
				data:  formData,
				type:  'POST',
				cache: false,
				success: function( data ) {
					// reload table with the new data
					$elem.closest( '.wpo-wcpbn-group' ).load( document.URL + ' .wpo-wcpbn-wrapper' );
					// unblock UI
					$( '#woocommerce-product-data' ).unblock();
				},
			} );
		} );
	}

	function wpoWcpbnEditOrderItem() {
		$( '#woocommerce-order-items' ).on( 'click', 'a.edit-order-item', function() {
			$( this ).closest( 'tr' ).find( '.wcpbn-item-batch-number-edit' ).show();

			// Not allow the number entered to be greater than the quantity of the item
			$( '.wcpbn-update-batch-number-quantity-sold' ).on( 'change', function() {
				let max = parseInt( $( this ).closest( 'tr' ).find( 'input.quantity' ).val() );
				let min = parseInt( $( this ).attr( 'min' ) );

				if ( $( this ).val() > max ) {
					$( this ).val( max );
				} else if ( $( this ).val() < min ) {
					$( this ).val( min );
				}       
			} );
		} );
	}

	function wpoWcpbnSupplierAutocomplete() {
		$( '.wpo-wcpbn-table-supplier' ).autocomplete( {
			source: function( request, response ) {
				// Fetch data
				$.ajax( {
					url:      wpo_wcpbn.ajaxurl,
					type:     'post',
					dataType: 'json',
					data: {
						action: 'supplier_names',
						search: request.term
					},
					success: function( data ) {
						response( data );
					}
				} );
			},
			select: function ( event, ui ) {
				// Set selection
				$( this ).val( ui.item.value );
				return false;
			}
		} );
	}

	function wpoWcpbnBatchNumberCheck() {
		$( '.wpo-wcpbn-table-batchnumber' )
		.autocomplete( {	
			delay:     0,
			autoFocus: false,
			source: function( request, response ) {
				let $input = this.element;

				// Fetch data
				$.ajax( {
					url:      wpo_wcpbn.ajaxurl,
					type:     'post',
					dataType: 'json',
					data: {
						action: 'batch_number_check',
						search: request.term
					},
					success: function( response ) {
						if ( response.success ) {
							$input.removeClass( 'ui-autocomplete-loading' ); // Hide spinner
							$input.attr( 'title', response.data.tooltip );

							if ( response.data.duplicated ) {
								$input.css( 'background-color', '#fbeaea' );
								// show tooltip
								$input.tooltip( {
									position: {
										my: 'center bottom-10',
										at: 'center top',
										using: function( position, feedback ) {
											$( this ).css( position );
											$( '<div>' )
											.addClass( 'arrow' )
											.addClass( feedback.vertical )
											.addClass( feedback.horizontal )
											.appendTo( this );
										}
									}
								} );
							} else {
								$input.css( 'background-color', '#ecf7ed' );
								// close tooltip
								if ( $input.is( ':ui-tooltip' ) ) {
									$input.tooltip( 'close' );
								}
							}
						}

						return;
					}
				} );
			},
			open: function( event, ui ) {
				$( '.ui-autocomplete' ).hide(); // Hide dropdown
				return;
			}
		} );
	}

	function wpoWcpbnSyncProductWcStock() {
		$( document ).on( 'click', '#wpo-wcpbn-sync-button', function( e ) {
			e.preventDefault();
			let $elem = $(this);
			let id    = $elem.parent().data( 'product_id' );

			// block UI
			$( '#woocommerce-product-data' ).block( {
				message: null,
				overlayCSS: {
					background: '#fff',
					opacity: 0.6,
				}
			} );

			// ajax request
			$.ajax( {
				url:   wpo_wcpbn.ajaxurl,
				data: {
					action:     'sync_product_wc_stock',
					product_id: id
				},
				type:  'POST',
				cache: false,
				success: function( response ) {
					let parentClass = $elem.parent().attr( 'class' );
					let classArr = parentClass.split( ' ' );

					// simple product
					if ( $.inArray( 'wpo-wcpbn-product-simple-stock-sync', classArr ) != -1 ) {
						// reload table with the new data
						$elem.closest( '.options_group' ).find( '.stock_fields' ).load( document.URL + ' .stock_fields .form-field' );
					// variation product
					} else if ( $.inArray( 'wpo-wcpbn-product-variation-stock-sync', classArr ) != -1 ) {
						// reload table with the new data
						location.reload();
					}

					// unblock UI
					$( '#woocommerce-product-data' ).unblock();
				},
			} );
		} );
	}

	// show/hide fields conditionally
	function wpoWcpbnDisplaySettingsConditionally() {
		$( "[data-show_for_option_name]" ).each( function( elem ) {
			let option_name = $(this).data( "show_for_option_name" );
			$(document).on( "change", "[name='"+option_name+"'], [name='"+option_name+"[]']", toggle_conditional_visibility );
			$( "[name='"+option_name+"'], [name='"+option_name+"[]']" ).trigger( "change" );
		} );
	}

	function toggle_conditional_visibility( event ) {
		let name     = $(this).prop('name').replace('[]',''); // normalize multiselect
		let value    = $(this).val();
		let checkbox = false;
		if ( $(this).is(":checkbox") ) {
			value = $(this).is(":checked");
			checkbox = true;
		}
		$( "[data-show_for_option_name='"+name+"']" ).each( function( elem ) {
			let show = false;
			if ( checkbox ) {
				show = value; // for checkboxes checked = show
			} else {
				let show_for = $(this).data( "show_for_option_values" );
				if ( Array.isArray( value ) ) { // Multiselect
					jQuery.each(value, function(index, item) {
						// if one option matches, show
						if ( $.inArray( item, show_for ) !== -1 ) {
							show = true;
						}
					});
				} else {
					if( value != null && value.indexOf( 'wcpdf_' ) >= 0 ) {
						value = 'documents';
					}
					show = $.inArray( value, show_for ) !== -1;
				}
			}

			let $row = $(this).is("tr") ? $(this) : $(this).closest("tr");
			if( show ) {
				$row.show();
			} else {
				$row.find( ':input' ).val( '' ).trigger( 'change' );
				$row.hide();
			}
		} );
	}

	function wpoWcpbnToggleExportDateRange() {
		$( '.wcpbn-card #wpo-wcpbn-export-batches-type, .wcpbn-card #wpo-wcpbn-export-orders-items-type' ).on( 'load change', function() {
			let $elem = $( this );
			$elem.closest( 'form' ).find( '.date-range' ).hide().find( 'input' ).attr( 'disabled', true );
			$elem.closest( 'form' ).find( '.order-number' ).hide().find( 'input' ).attr( 'disabled', true );
			if ( $elem.val() == 'date-range' ) {
				$elem.closest( 'form' ).find( '.date-range' ).show().find( 'input' ).attr( 'disabled', false );
				$elem.closest( 'form' ).find( '.order-number' ).hide().find( 'input' ).attr( 'disabled', true );
			} else {
				$elem.closest( 'form' ).find( '.date-range' ).hide().find( 'input' ).attr( 'disabled', true );
				$elem.closest( 'form' ).find( '.order-number' ).show().find( 'input' ).attr( 'disabled', false );
			}
		} ).trigger( 'change' );
	}

	// Add a class to edited fields in the batch table.
	$( '.wp-list-table.batches' ).on( 'input', 'input[name^="batches"]', function () {
		const input_field = $( this );
		const td 		  = input_field.closest( 'td' );
		const row 		  = input_field.closest( 'tr' );
		const checkbox    = row.find( 'input[type="checkbox"][name^="batch_id"]' );

		if ( input_field.val() != input_field.data( 'value' ) ) {
			td.addClass( 'wpo-wcpbn-edited' );
		} else {
			td.removeClass( 'wpo-wcpbn-edited' );
		}

		// Check or uncheck the checkbox of the row based on the row's edit state.
		checkbox.prop( 'checked', is_row_edited( row ) );

		// Check if any row has been edited to update the bulk option and apply button class.
		if ( is_any_row_edited() ) {
			$( '#bulk-action-selector-top' ).val( 'update' );
			$( '#doaction' ).addClass( 'button-primary' );
		} else {
			$( '#bulk-action-selector-top' ).val( -1 );
			$( '#doaction' ).removeClass( 'button-primary' );
		}
	} );

	function is_row_edited( row ) {
		return row.find( 'input[name^="batches"]' ).is( function () {
			return $( this ).val() != $( this ).data( 'value' );
		} );
	}

	function is_any_row_edited() {
		return $( '.wp-list-table.batches tr' ).is( function() {
			return is_row_edited( $( this ) );
		} );
	}

} );