<?php
/**
 * Batch Numbers DB Class
 *
 * @since 1.0.0
 */

namespace WPO\WC\Product_Batch_Numbers\Database;

/**
 * If this file is called directly, abort.
 *
 * @since 1.0.0
 */
if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if ( ! class_exists( 'wcpbnDbBatchNumbers' ) ) {

	class wcpbnDbBatchNumbers extends wcpbnAbstractDb {

		protected static $_instance = null;

		/**
		 * Instance
		 */
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		/**
		 * Construct
		 */
		public function __construct() {
			parent::__construct( 'batch_numbers' );
			
			$this->default_special_keys = array( 'search', 'limit', 'offset', 'order', 'order_by', 'shared_ids' );
		}

		/**
		 * Create the plugin database table.
		 *
		 * @return array|object|null
		 */
		protected function create_table() {
			$charset_collate = $this->wpdb->get_charset_collate();

			$sql = "CREATE TABLE " . $this->get_full_table_name() . " (
				id bigint(20) NOT NULL AUTO_INCREMENT,
				date_created datetime DEFAULT '1000-01-01 00:00:00' NOT NULL,
				date_expiry datetime DEFAULT '1000-01-01 00:00:00' NULL,
				user_id smallint(5) NOT NULL,
				product_id mediumint(9) NOT NULL,
				variation_id mediumint(9) NULL,
				batch_number varchar(200) NOT NULL,
				supplier varchar(200) DEFAULT '',
				location varchar(200) DEFAULT '',
				quantity decimal(20,4) NOT NULL,
				quantity_available decimal(20,4) NOT NULL,
				units smallint(5) DEFAULT 1 NOT NULL,
				status varchar(200) DEFAULT 'active' NOT NULL,
				order_ids longtext NULL,
				PRIMARY KEY  (id)
			) $charset_collate;";

			require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
			dbDelta( $sql );

			$this->catch_db_object_errors( $this->wpdb );

			return;
		}

		/**
		 * Get date keys.
		 *
		 * @return array()
		 */
		public function get_date_keys() {
			return array( 'date_created', 'date_expiry' );
		}

		/**
		 * Get supplier by name.
		 *
		 * @return array|object|null
		 */
		public function get_supplier_by_name( $name ) {
			if ( empty( $name ) ) {
				return null;
			}

			return $this->wpdb->get_results( "SELECT DISTINCT supplier FROM " . $this->get_full_table_name() . " WHERE supplier LIKE '%{$name}%'" );
		}

		/**
		 * Get batch number order IDs.
		 *
		 * @return array|object|null
		 */
		public function get_batch_order_ids( $batch_id ) {
			if ( empty( $batch_id ) ) {
				return null;
			}

			return $this->wpdb->get_results( "SELECT order_ids FROM " . $this->get_full_table_name() . " WHERE id LIKE '{$batch_id}'" );
		}

		/**
		 * Get batches by order id.
		 * 
		 * @param $order_id
		 *
		 * @return array|object|null
		 */
		public function get_batches_by_order_id( $order_id ) {
			$regex = "(?<![\d]){$order_id}(?![\d])";
			
			return $this->wpdb->get_results( 
				$this->wpdb->prepare(
					"SELECT * FROM " . $this->get_full_table_name() . " WHERE order_ids REGEXP %s",
					$regex
				)
			);			
		}
	}

}