<?php
/**
 * Purchase Orders DB Class
 */

namespace WPO\WC\Product_Batch_Numbers\Database;

/**
 * If this file is called directly, abort.
 */
if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if ( ! class_exists( 'wcpbnDbPurchaseOrders' ) ) {

	class wcpbnDbPurchaseOrders extends wcpbnAbstractDb {

		protected static $_instance  = null;

		/**
		 * Instance
		 */
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		/**
		 * Construct
		 */
		public function __construct() {
			parent::__construct( 'purchase_orders' );
		}

		/**
		 * Create the plugin database table.
		 *
		 * @return void
		 */
		protected function create_table() {
			$charset_collate = $this->wpdb->get_charset_collate();

			$sql = "CREATE TABLE " . $this->get_full_table_name() . " (
				id bigint(20) NOT NULL AUTO_INCREMENT,
				date_created datetime DEFAULT '1000-01-01 00:00:00' NOT NULL,
				batch_id varchar(200) NOT NULL,
				po_number varchar(200) NOT NULL,
				supplier varchar(200) DEFAULT '',
				cost LONGTEXT NULL,
				regular_price LONGTEXT NULL,
				sale_price LONGTEXT NULL,
				price_margin LONGTEXT NULL,
				price_discount LONGTEXT NULL,
				PRIMARY KEY  (id)
			) $charset_collate;";

			require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
			dbDelta( $sql );

			$this->catch_db_object_errors( $this->wpdb );
		}
		
		public function get_row_by_batch_id( $batch_id ) {
			if ( empty( $batch_id ) || ! is_numeric( $batch_id ) ) {
				return null;
			}
			
			$sql   = "SELECT * FROM {$this->get_full_table_name()} WHERE batch_id = '%s'";
			$query = $this->wpdb->prepare( $sql, $batch_id );

			return $this->wpdb->get_results( $query );
		}
		
		public function get_rows_from_batch_ids( array $batch_ids ) {
			if ( empty( $batch_ids ) ) {
				return null;
			}
			
			$ids         = array_map( 'intval', $batch_ids );
			$placeholder = implode( ',', array_fill( 0, count( $ids ), '%s' ) );
			$sql         = "SELECT * FROM {$this->get_full_table_name()} WHERE batch_id IN ($placeholder)";
			$query       = $this->wpdb->prepare( $sql, $ids );
			
			return $this->wpdb->get_results( $query );
		}

	}

}