<?php
/**
 * Shared Products DB Class
 */

namespace WPO\WC\Product_Batch_Numbers\Database;

/**
 * If this file is called directly, abort.
 */
if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if ( ! class_exists( 'wcpbnDbSharedProducts' ) ) {

	class wcpbnDbSharedProducts extends wcpbnAbstractDb {

		protected static $_instance  = null;

		/**
		 * Instance
		 */
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		/**
		 * Construct
		 */
		public function __construct() {
			parent::__construct( 'shared_products' );
			
			add_action( 'wcpbn_schedule_delete_duplicate_shared_products', array( $this, 'delete_duplicates' ) );
		}

		/**
		 * Create the plugin database table.
		 *
		 * @return void
		 */
		protected function create_table() {
			$charset_collate = $this->wpdb->get_charset_collate();

			$sql = "CREATE TABLE " . $this->get_full_table_name() . " (
				id bigint(20) NOT NULL AUTO_INCREMENT,
				date_created datetime DEFAULT '1000-01-01 00:00:00' NOT NULL,
				batch_id varchar(200) NOT NULL,
				product_id varchar(200) NOT NULL,
				PRIMARY KEY  (id)
			) $charset_collate;";

			require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
			dbDelta( $sql );

			$this->catch_db_object_errors( $this->wpdb );

			return;
		}
		
		public function join_callback( $sql, $args, $table_name ) {
			if ( ! empty( $table_name ) && ! empty( $args['shared_ids'] ) && is_array( $args['shared_ids'] ) ) {				
				$sql .= " WHERE EXISTS (SELECT * FROM {$this->get_full_table_name()} WHERE {$table_name}.id = {$this->get_full_table_name()}.batch_id AND {$this->get_full_table_name()}.product_id IN (" . implode( ',', array_map( 'intval', $args['shared_ids'] ) ) . "))";
			}
			
			return $sql;
		}
		
		/**
		 * Deletes duplicate rows keeping the highest ID
		 *
		 * @return void
		 */
		public function delete_duplicates() {
			$query = $this->wpdb->query( "DELETE b1 FROM {$this->get_full_table_name()} b1 INNER JOIN {$this->get_full_table_name()} b2 WHERE b1.id < b2.id AND b1.batch_id = b2.batch_id AND b1.product_id = b2.product_id" );
			WPO_WCPBN()->classes['logger']->log( "{$query} duplicated rows deleted from the shared products database table.", 'info' );
		}

	}

}