<?php

namespace WPO\WC\Product_Batch_Numbers\Settings;

use WPO\WC\Product_Batch_Numbers\wcpbnBatchNumberStatus;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if ( ! class_exists( 'wcpbnSettingsExport' ) ) {

	class wcpbnSettingsExport extends wcpbnAbstractSettings {

		protected static $_instance = null;

		/**
		 * Instance
		 */
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		/**
		 * Construct
		 */
		public function __construct() {
			$this->id = 'export';

			add_action( 'wpo_wcpbn_after_settings_tabs', array( $this, 'wpai_addon_notice' ), 1, 2 );
			add_action( 'wpo_wcpbn_settings_tab_content', array( $this, 'tab_content' ), 10, 2 );
		}

		/**
		 * Populate the settings page tab with data
		 *
		 * @param  string  $current_tab
		 * @param  string  $current_section
		 * 
		 * @return void
		 */
		public function tab_content( $current_tab, $current_section ) {
			if ( $current_tab != $this->id ) {
				return;
			}
			?>
			<div class="wpo_wcpbn_settings_tab">
			<?php
				do_action( 'wpo_wcpbn_before_settings_export', $this->get_option_key() );
				$this->export_actions();
				do_action( 'wpo_wcpbn_after_settings_export', $this->get_option_key() );
			?>
			</div>
			<?php
		}

		/**
		 * Populate the export sections
		 *
		 * @param  string  $current_section
		 * 
		 * @return void
		 */
		public function export_actions() {
			?>
			<div class="wcpbn-export">
				<div class="wcpbn-export__main">
					<div class="wcpbn-export-container">
						<!-- Export batch customers to CSV -->
						<div class="wcpbn-card">
							<div class="wcpbn-card__header">
								<h2><?php _e( 'Export batch customers', 'woocommerce-product-batch-numbers' ); ?></h2>
								<h3><?php _e( 'Export all your customers that purchased a specific batch number.', 'woocommerce-product-batch-numbers' ); ?></h3>
							</div>
							<div class="wcpbn-card__body">
								<div class="wcpbn-card__body-content">
									<div class="wcpbn-card__body-content-item">
										<form id="wcpbn-export-customers-form" action="<?= admin_url( 'admin-ajax.php' ); ?>" enctype="multipart/form-data" method="post">
											<?php wp_nonce_field( 'wpo_wcpbn_export', 'security' ); ?>
											<div style="display: inline-block;">
												<label><strong><?php _e( 'From', 'woocommerce-product-batch-numbers' ); ?></strong></label><br>
												<input type="date" name="from" required>
											</div>
											<div style="display: inline-block; margin-left: 10px;">
												<label><strong><?php _e( 'To', 'woocommerce-product-batch-numbers' ); ?></strong></label><br>
												<input type="date" name="to" required>
											</div>
											<div>
												<label><strong><?php _e( 'Batch ID', 'woocommerce-product-batch-numbers' ); ?></strong></label><br>
												<input type="number" name="batch-id" required><span style="margin-left: 10px;"><a href="<?= admin_url( 'edit.php?post_type=product&page=wpo-batch-numbers&tab=overview' ); ?>" target="_blank"><?php _e( 'Display in the Overview tab', 'woocommerce-product-batch-numbers' ) ?> &rarr;</a></span>
											</div>
											<div>
												<input type="hidden" name="action" value="wpo-wcpbn-export-customers">
											</div>
											<div>
												<button class="button wcpbn-export-customers-submit" type="submit"><?php _e( 'Export', 'woocommerce-product-batch-numbers' ); ?></button>
											</div>
										</form>
									</div>
								</div>
							</div>
						</div>
						<!-- Export orders items with batch -->
						<div class="wcpbn-card">
							<div class="wcpbn-card__header">
								<h2><?php _e( 'Export Orders Items with Batches', 'woocommerce-product-batch-numbers' ); ?></h2>
								<h3><?php _e( 'Export orders items with batches data from a determined date range.', 'woocommerce-product-batch-numbers' ); ?></h3>
							</div>
							<div class="wcpbn-card__body">
								<div class="wcpbn-card__body-content">
									<div class="wcpbn-card__body-content-item">
										<form id="wpo-wcpbn-export-orders-items-batches-form" action="<?= admin_url( 'admin-ajax.php' ); ?>" enctype="multipart/form-data" method="post">
											<?php wp_nonce_field( 'wpo_wcpbn_export', 'security' ); ?>
											<div>
												<label><strong><?php _e( 'Type', 'woocommerce-product-batch-numbers' ); ?></strong></label><br>
												<select id="wpo-wcpbn-export-orders-items-type" name="type" style="margin: 10px 0;" required>
													<option value="" disabled><?php _e( 'Select...', 'woocommerce-product-batch-numbers' ); ?></option>
													<option value="date-range"><?php _e( 'Export all', 'woocommerce-product-batch-numbers' ); ?></option>
													<option value="order-number"><?php _e( 'By order number', 'woocommerce-product-batch-numbers' ); ?></option>
												</select>
											</div>
											<div class="date-range">
												<div style="display: inline-block;">
													<label><strong><?php _e( 'From', 'woocommerce-product-batch-numbers' ); ?></strong></label><br>
													<input type="date" name="from" required>
												</div>
												<div style="display: inline-block; margin-left: 10px;">
													<label><strong><?php _e( 'To', 'woocommerce-product-batch-numbers' ); ?></strong></label><br>
													<input type="date" name="to" required>
												</div>
											</div>
											<div class="order-number">
												<label><strong><?php _e( 'Order Number', 'woocommerce-product-batch-numbers' ); ?></strong></label><br>
												<input type="number" name="order-number"><span style="margin-left: 10px;"><a href="<?= admin_url( 'edit.php?post_type=shop_order' ); ?>" target="_blank"><?php _e( 'See in the Orders table', 'woocommerce-product-batch-numbers' ); ?> &rarr;</a></span>
											</div>
											<div>
												<input type="hidden" name="action" value="wpo-wcpbn-export-orders-items-batches">
											</div>
											<div>
												<button class="button wcpbn-export-orders-items-batches-submit" type="submit"><?php _e( 'Export', 'woocommerce-product-batch-numbers' ); ?></button>
											</div>
										</form>
									</div>
								</div>
							</div>
						</div>
						<!-- Export batches to CSV -->
						<div class="wcpbn-card">
							<div class="wcpbn-card__header">
								<h2><?php _e( 'Export batches', 'woocommerce-product-batch-numbers' ); ?></h2>
								<h3><?php _e( 'Export saved batches data from a determined date range.', 'woocommerce-product-batch-numbers' ); ?></h3>
							</div>
							<div class="wcpbn-card__body">
								<div class="wcpbn-card__body-content">
									<div class="wcpbn-card__body-content-item">
										<form id="wcpbn-export-batches-form" action="<?= admin_url( 'admin-ajax.php' ); ?>" enctype="multipart/form-data" method="post">
											<?php wp_nonce_field( 'wpo_wcpbn_export', 'security' ); ?>
											<div>
												<label><strong><?php _e( 'Type', 'woocommerce-product-batch-numbers' ); ?></strong></label><br>
												<select id="wpo-wcpbn-export-batches-type" name="type" style="margin: 10px 0;">
													<option value="" disabled><?php _e( 'Select...', 'woocommerce-product-batch-numbers' ); ?></option>
													<option value="all"><?php _e( 'Export all', 'woocommerce-product-batch-numbers' ); ?></option>
													<option value="date-range"><?php _e( 'Export by date range', 'woocommerce-product-batch-numbers' ); ?></option>
												</select>
											</div>
											<div class="date-range">
												<div style="display: inline-block;">
													<label><strong><?php _e( 'From', 'woocommerce-product-batch-numbers' ); ?></strong></label><br>
													<input type="date" name="from" required>
												</div>
												<div style="display: inline-block; margin-left: 10px;">
													<label><strong><?php _e( 'To', 'woocommerce-product-batch-numbers' ); ?></strong></label><br>
													<input type="date" name="to" required>
												</div>
											</div>
											<div>
												<label><strong><?php _e( 'Status', 'woocommerce-product-batch-numbers' ); ?></strong></label><br>
												<select id="wpo-wcpbn-export-batches-status" name="status" style="margin: 10px 0;">
													<option value="" disabled><?php _e( 'Select...', 'woocommerce-product-batch-numbers' ); ?></option>
													<option value="all"><?php _e( 'All', 'woocommerce-product-batch-numbers' ); ?></option>
													<?php
														$status = wcpbnBatchNumberStatus::batch_number_status();
														foreach ( $status as $slug => $name ) :
													?>
														<option value="<?= $slug; ?>"><?= $name; ?></option>
													<?php endforeach; ?>
												</select>
											</div>
											<?php if ( class_exists( 'WPO_WCPBN_WPAI' ) ) : // WPAI add-on ?>
												<div>
													<table style="border:1px solid #eee; padding:10px;">
														<tbody>
															<tr>
																<td>
																	<img src="<?= WPO_WCPBN()->plugin_dir_url.'assets/images/wpai-logo.svg'; ?>" alt="WP All Import Logo">
																</td>
																<td style="font-size:9pt; font-weight:bold; padding-left:10px;">
																	<label>
																		<?php
																			printf(
																				/* translators: WP All Import */
																				__( 'Include support for <u>%s</u>', 'woocommerce-product-batch-numbers' ),
																				'WP All Import'
																			);
																		?>
																	</label>
																	<input type="checkbox" id="wpo-wcpbn-export-batches-wpai" name="wpai" style="margin-left:5px;" >
																</td>
															</tr>
														</tbody>
													</table>
												</div>
											<?php endif; ?>
											<div>
												<input type="hidden" name="action" value="wpo-wcpbn-export-batches">
											</div>
											<div>
												<button class="button wcpbn-export-batches-submit" type="submit"><?php _e( 'Export', 'woocommerce-product-batch-numbers' ); ?></button>
											</div>
										</form>
									</div>
								</div>
							</div>
						</div>
						<!-- Export batches by order id to CSV -->
						<div class="wcpbn-card">
							<div class="wcpbn-card__header">
								<h2><?php _e( 'Export Order Batches', 'woocommerce-product-batch-numbers' ); ?></h2>
								<h3><?php _e( 'Export batch numbers of a specific order number.', 'woocommerce-product-batch-numbers' ); ?></h3>
							</div>
							<div class="wcpbn-card__body">
								<div class="wcpbn-card__body-content">
									<div class="wcpbn-card__body-content-item">
										<form id="wcpbn-export-order-specific-batches-form" action="<?= admin_url( 'admin-ajax.php' ); ?>" enctype="multipart/form-data" method="post">
											<?php wp_nonce_field( 'wpo_wcpbn_export', 'security' ); ?>
											<div>
												<label><strong><?php _e( 'Order Number', 'woocommerce-product-batch-numbers' ); ?></strong></label><br>
												<input type="number" name="order-number" required><span style="margin-left: 10px;"><a href="<?= admin_url( 'edit.php?post_type=shop_order' ); ?>" target="_blank"><?php _e( 'See in the Orders table', 'woocommerce-product-batch-numbers' ); ?> &rarr;</a></span>
											</div>
											<div>
												<input type="hidden" name="action" value="wpo-wcpbn-export-order-batches">
											</div>
											<div>
												<button class="button wcpbn-export-order-specific-batches-submit" type="submit"><?php _e( 'Export', 'woocommerce-product-batch-numbers' ); ?></button>
											</div>
										</form>
									</div>
								</div>
							</div>
						</div>
					</div>
				</div>
			</div>
			<?php
		}

		/**
		 * Displays WPAI notice
		 */
		public function wpai_addon_notice( $current_tab, $current_section ) {
			if ( ! class_exists( 'WPO_WCPBN_WPAI' ) && $current_tab == 'export' ) :
			?>
				<div class="notice notice-info inline">
					<p>
						<?php
							printf(
								/* translators: here */
								__( "We have a small add-on to import/update batches using <strong>WP All Import</strong>. If you're interested please drop us a message %s.", 'woocommerce-product-batch-numbers' ),
								'<a href="https://wpovernight.com/contact/" target="_blank">'.__( 'here', 'woocommerce-product-batch-numbers' ).'</a>'
							);
						?>
					</p>
				</div>
			<?php
			endif;
		}

	}

}