<?php

namespace WPO\WC\Product_Batch_Numbers\Settings;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if ( ! class_exists( 'wcpbnSettingsOptions' ) ) {

	class wcpbnSettingsOptions extends wcpbnAbstractSettings {

		protected static $_instance = null;

		/**
		 * @var array
		 */
		private $tabs;
		/**
		 * Instance
		 */
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		/**
		 * Construct.
		 */
		public function __construct() {
			$this->id   = 'options';
			$this->tabs = apply_filters( 'wpo_wcpbn_settings_tabs', array(
				'general'  => __( 'General', 'woocommerce-product-batch-numbers' ),
				'po'       => __( 'PO', 'woocommerce-product-batch-numbers' ),
				'display'  => __( 'Display', 'woocommerce-product-batch-numbers' ),
				'email'    => __( 'Email', 'woocommerce-product-batch-numbers' ),
				'advanced' => __( 'Advanced', 'woocommerce-product-batch-numbers' ),
			) );

			$this->init_form_fields();
			$this->init_settings();

			add_action( 'admin_init', array( $this, 'init_form_fields' ), 5 ); // refresh to make sure third party functions we use can be called
			add_action( 'admin_init', array( $this, 'register_settings' ) );
			add_action( 'admin_init', array( $this, 'schedule_recurring_actions' ) );
			add_action( 'wpo_wcpbn_settings_tab_content', array( $this, 'tab_content' ), 10, 2 );
		}

		public function get_option_key() {
			return parent::get_option_key() . '_' . $this->get_active_tab();
		}

		public function init_settings() {
			// When the settings page form is submitted, or a specific settings tab is requested.
			if ( ( isset( $_REQUEST['action'] ) && 'update' === $_REQUEST['action'] ) || isset( $_REQUEST['section'] ) ) {
				$this->settings = get_option( $this->get_option_key(), array() );
			}
			// In cases where a specific tab or update action isn't defined, aggregate settings from all tabs.
			// This branch is typically executed when the parent class method 'get_option()' is called.
			else {
				foreach ( $this->tabs as $tab => $title ) {
					$this->settings = array_merge( $this->settings, get_option( parent::get_option_key() . '_' . $tab, array() ) );
				}
			}

			// If there are no settings defined, use defaults.
			if ( empty( $this->settings ) ) {
				$form_fields    = $this->get_form_fields();
				$this->settings = array_merge( array_fill_keys( array_keys( $form_fields ), '' ), wp_list_pluck( $form_fields, 'default' ) );
			}
		}

		/**
		 * Populates the settings page tab with data
		 *
		 * @param  string  $current_tab
		 * @param  string  $current_section
		 * 
		 * @return void
		 */
		public function tab_content( $current_tab, $current_section ) {
			if ( $current_tab !== $this->id ) {
				return;
			}

			$option_key = $this->get_option_key();

			?>
			<div class="wpo_wcpbn_settings_tab">
				<form id="<?php echo $option_key; ?>" class="<?php echo $option_key; ?>" method="post" action="options.php">
					<div class="wpo_wcpbn_settings_sections">
						<h2 class="nav-tab-wrapper">
							<?php
							foreach ( $this->tabs as $slug => $title ) {
								$active = ( $slug === $this->get_active_tab() ) ? 'nav-tab-active' : '';
								printf( '<a href="%1$s" class="nav-tab nav-tab-%2$s %3$s">%4$s</a>', esc_url( add_query_arg( 'section', $slug ) ), esc_attr( $slug ), $active, esc_html( $title ) );
							}
							?>
						</h2>
					</div>
					<div class="wpo_wcpbn_settings_tab_form_fields">
					<?php
					do_action( 'wpo_wcpbn_before_settings_options', $option_key );
					$this->admin_options();
					do_action( 'wpo_wcpbn_after_settings_options', $option_key );
					?>
					</div>
					<div class="wpo_wcpbn_settings_tab_form_submit">
					<?php
					submit_button();
					?>
					</div>
				</form>
			</div>
			<?php
		}

		/**
		 * Passes the settings form fields to the 'form_fields' property
		 *
		 * @return void
		 */
		public function init_form_fields() {
			$active_tab = $this->get_active_tab();
			if ( method_exists( $this, $active_tab . '_form_fields' ) ) {
				/** @uses general_form_fields(), po_form_fields(), display_form_fields(), email_form_fields(), advanced_form_fields() */
				$form_fields       = call_user_func( array( $this, $active_tab . '_form_fields' ) );
				$this->form_fields = apply_filters( 'wpo_wcpbn_settings_options_args', $form_fields, $active_tab );
			}
		}

		public function wc_frontend_product_page_hooks( $keys_only = false ) {
			$hooks = apply_filters( 'wpo_wcpbn_wc_frontend_product_page_hooks', array(
				'woocommerce_single_product_summary'      => __( 'Single product summary', 'woocommerce-product-batch-numbers' ),
				'woocommerce_before_add_to_cart_form'     => __( 'Before the add to cart form', 'woocommerce-product-batch-numbers' ),
				'woocommerce_after_add_to_cart_form'      => __( 'After the add to cart form', 'woocommerce-product-batch-numbers' ),
				'woocommerce_before_add_to_cart_button'   => __( 'Before the add to cart button', 'woocommerce-product-batch-numbers' ),
				'woocommerce_after_add_to_cart_button'    => __( 'After the add to cart button', 'woocommerce-product-batch-numbers' ),
				'woocommerce_before_add_to_cart_quantity' => __( 'Before the quantity field in the add to cart form', 'woocommerce-product-batch-numbers' ),
				'woocommerce_after_add_to_cart_quantity'  => __( 'After the quantity field in the add to cart form', 'woocommerce-product-batch-numbers' ),
				'woocommerce_product_meta_start'          => __( 'Before product meta data', 'woocommerce-product-batch-numbers' ),
				'woocommerce_product_meta_end'            => __( 'After product meta data', 'woocommerce-product-batch-numbers' ),
			) );

			if ( $keys_only ) {
				return array_keys( $hooks );
			} else {
				return $hooks;
			}
		}

		public function generate_display_batch_data_in_product_pages_time_window_html( $data ) {
			$key       = $data['key'];
			$field_key = $this->get_field_key( $key );

			$options = $this->get_option( $key, array(
				'enable' => '',
				'from'   => '',
				'to'     => '',
			) );

			$defaults  = array(
				'title'             => '',
				'label'             => '',
				'disabled'          => false,
				'readonly'          => false,
				'class'             => '',
				'css'               => '',
				'desc_tip'          => false,
				'description'       => '',
				'custom_attributes' => array(),
				'size'              => '',
				'secondary'         => false,
			);

			$data = wp_parse_args( $data, $defaults );

			if ( ! $data['label'] ) {
				$data['label'] = $data['title'];
			}

			if ( ! empty( $data['size'] ) ) {
				$data['css'] .= ';width:auto;';
			}
			?>
			<fieldset>
				<?php if( $data['secondary'] && ! empty( $data['title'] ) ) : ?>
					<label for="<?php echo esc_attr( $field_key ); ?>"><strong><?php echo wp_kses_post( $data['title'] ); ?>:</strong></label><br>
				<?php endif; ?>
				<input <?php disabled( $data['disabled'], true ); ?> <?php $this->wp_readonly( $data['readonly'] ); ?> class="<?php echo esc_attr( $data['class'] ); ?>_enable" type="checkbox" name="<?php echo esc_attr( $field_key ); ?>[enable]" id="<?php echo esc_attr( $field_key ); ?>[enable]" value="1" <?php checked( $options['enable'], 'yes' ); ?> <?php echo $this->get_custom_attribute_html( $data ); // WPCS: XSS ok. ?> />
				<span>
					<?php
					$from_input = sprintf(
						'<input %1$s %2$s class="%3$s_from" type="number" name="%4$s[from]" id="%4$s[from]" style="%5$s" value="%6$s" %7$s size="1" />',
						disabled( $data['disabled'], true, false ),
						$this->wp_readonly( $data['readonly'], true, false ),
						esc_attr( $data['class'] ),
						esc_attr( $field_key ),
						esc_attr( $data['css'] ),
						$options['from'],
						$this->get_custom_attribute_html( $data )
					);
					$to_input = sprintf(
						'<input %1$s %2$s class="%3$s_to" type="number" name="%4$s[to]" id="%4$s[to]" style="%5$s" value="%6$s" %7$s size="1" />',
						disabled( $data['disabled'], true, false ),
						$this->wp_readonly( $data['readonly'], true, false ),
						esc_attr( $data['class'] ),
						esc_attr( $field_key ),
						esc_attr( $data['css'] ),
						$options['to'],
						$this->get_custom_attribute_html( $data )
					);
					printf(
						/* translators: input fields (number of days) */
						esc_html__( 'only show from %1$s days until %2$s days before expiry', 'woocommerce-product-batch-numbers' ),
						$from_input,
						$to_input
					);
					?>
				</span>
				<?php echo $this->get_description_html( $data ); // WPCS: XSS ok. ?>
			</fieldset>
			<?php
		}

		public function validate_display_batch_data_in_product_pages_time_window_field( $key, $value ) {
			$empty_defaults = array(
				'enable' => '',
				'from'   => '',
				'to'     => '',
			);
			$value           = is_array( $value ) ? array_merge( $empty_defaults, $value ) : $empty_defaults;
			$value           = array_intersect_key( $value, $empty_defaults );
			$value['enable'] = $this->validate_checkbox_field( $key, $value['enable'] );
			$value['from']   = $this->validate_text_field( $key, $value['from'] );
			$value['to']     = $this->validate_text_field( $key, $value['to'] );

			return $value;
		}

		public function get_batch_dates() {
			return apply_filters( 'wpo_wcpbn_batch_dates', [
				'expiry_date'      => __( 'Expiry Date', 'woocommerce-product-batch-numbers' ),
				'best_before_date' => __( 'Best Before Date', 'woocommerce-product-batch-numbers' ),
			] );
		}

		public function get_batch_date_label( $date_slug ) {
			$dates = $this->get_batch_dates();

			if ( ! empty( $date_slug ) && isset( $dates[$date_slug] ) ) {
				$label = $dates[$date_slug];
			} else {
				$label = $dates['expiry_date'];
			}

			return $label;
		}

		public function schedule_recurring_actions() {
			if ( ! function_exists( 'as_schedule_recurring_action' ) ) {
				return;
			}

			$timestamp           = apply_filters( 'wpo_wcpbn_settings_options_recurring_actions_timestamp', strtotime( 'tomorrow' ) );
			$interval_in_seconds = apply_filters( 'wpo_wcpbn_settings_options_recurring_actions_interval_in_seconds', DAY_IN_SECONDS );

			if ( false === as_next_scheduled_action( 'wcpbn_schedule_delete_duplicate_shared_products' ) && wc_string_to_bool( $this->get_option( 'delete_duplicate_shared_products' ) ) ) {
				as_schedule_recurring_action( $timestamp, $interval_in_seconds, 'wcpbn_schedule_delete_duplicate_shared_products' );
			}
		}

		/**
		 * Returns an array settings for the general tab.
		 *
		 * @return array
		 */
		private function general_form_fields() {
			$form_fields = array(
				'section_general'                => array(
					'title'   => '',
					'section' => 'general',
					'type'    => 'title',
					'id'      => 'title',
				),
				'batch_prioritization'           => array(
					'title'       => __( 'Batch prioritization', 'woocommerce-product-batch-numbers' ),
					'type'        => 'select',
					'options'     => array(
						'date_created'       => __( 'Order of addition', 'woocommerce-product-batch-numbers' ),
						'date_expiry'        => __( 'Order of expiration', 'woocommerce-product-batch-numbers' ),
						'quantity_available' => __( 'Available stock', 'woocommerce-product-batch-numbers' ),
					),
					'default'     => 'quantity_available',
					'description' => __( "Batch allocation will follow this option's value for prioritization.", 'woocommerce-product-batch-numbers' ),
					'css'         => 'min-width: 400px;',
				),
				'global_expiration_offset'       => array(
					'title'       => __( 'Batch expiration offset (days)', 'woocommerce-product-batch-numbers' ),
					'default'     => apply_filters( 'wpo_wcpbn_default_offset_value', '30' ),
					'type'        => 'number',
					'description' => __( 'Set a global number of days before a batch gets expired. This will be used to mark the "Expiring date" with a different color in the Overview table and also for email notifications.', 'woocommerce-product-batch-numbers' ) . '<br><strong>' . __( 'Note:', 'woocommerce-product-batch-numbers' ) . '</strong> ' . __( 'Can be overridden per product', 'woocommerce-product-batch-numbers' ),
					'css'         => 'min-width: 400px;',
				),
				'archive_batches'                => array(
					'title'       => __( 'Archive batches', 'woocommerce-product-batch-numbers' ),
					'type'        => 'select',
					'description' => __( 'Automatically archive batches. (Runs once a day)', 'woocommerce-product-batch-numbers' ),
					'options'     => array(
						'no'        => __( 'No', 'woocommerce-product-batch-numbers' ),
						'expired'   => __( 'Expired', 'woocommerce-product-batch-numbers' ),
						'on_offset' => __( 'On offset', 'woocommerce-product-batch-numbers' ),
					),
				),
				'assign_expired'                 => array(
					'title'       => __( 'Assign as expired', 'woocommerce-product-batch-numbers' ),
					'type'        => 'checkbox',
					'description' => __( 'Allow assigning expired batches to order items.', 'woocommerce-product-batch-numbers' ),
				),
				'sync_wc_stock'                  => array(
					'title'       => __( 'Sync with WooCommerce stock', 'woocommerce-product-batch-numbers' ),
					'type'        => 'checkbox',
					'description' => __( 'The WooCommerce stock is linked to the total batch stock and automatically updated when you add, update or remove batches.', 'woocommerce-product-batch-numbers' ),
				),
				'stock_locations'                => array(
					'title'       => __( 'Stock locations', 'woocommerce-product-batch-numbers' ),
					'type'        => 'checkbox',
					'description' => __( 'Enable this if you want to add stock locations to batches. This is pretty useful for businesses that store products.', 'woocommerce-product-batch-numbers' ),
				),
				'delete_batch_on_product_delete' => array(
					'title'       => __( 'Delete batches on product deletion', 'woocommerce-product-batch-numbers' ),
					'type'        => 'checkbox',
					'description' => __( 'Enable this if you want to delete assigned batches when deleting their products. This only happens when you delete products from trash.', 'woocommerce-product-batch-numbers' ),
				),
				'custom_date_format'             => array(
					'title'       => __( 'Custom date format', 'woocommerce-product-batch-numbers' ),
					'type'        => 'text',
					'description' => sprintf(
					/* translators: here link */
						__( 'Override the default plugin date format. Find examples %s.', 'woocommerce-product-batch-numbers' ),
						'<a href="https://wordpress.org/support/article/formatting-date-and-time/" target="_blank">' . __( 'here', 'woocommerce-product-batch-numbers' ) . '</a>'
					),
					'default'     => wc_date_format(),
				),
				'batch_date_label'               => array(
					'title'   => __( 'Batch date label', 'woocommerce-product-batch-numbers' ),
					'type'    => 'select',
					'options' => $this->get_batch_dates(),
					'default' => 'expiry_date',
				),
			);

			return $form_fields;
		}

		/**
		 * Returns an array settings for the PO tab.
		 * 
		 * @return array
		 */
		private function po_form_fields() {
			return array(
				'section_po'                      => array(
					'title'   => '',
					'section' => 'po',
					'type'    => 'title',
					'id'      => 'title',
				),
				'use_po_prices'                   => array(
					'title'       => __( 'Override product prices', 'woocommerce-product-batch-numbers' ),
					'type'        => 'checkbox',
					'description' => __( 'Will override the default product regular and sale prices with the batch PO values, when they exist.', 'woocommerce-product-batch-numbers' ),
				),
				'po_price_prioritization'         => array(
					'title'             => __( 'Price prioritization', 'woocommerce-product-batch-numbers' ),
					'type'              => 'select',
					'options'           => array(
						'next_batch'            => __( 'Next batch with PO', 'woocommerce-product-batch-numbers' ),
						'highest_regular_price' => __( 'Highest PO regular price', 'woocommerce-product-batch-numbers' ),
						'lowest_regular_price'  => __( 'Lowest PO regular price', 'woocommerce-product-batch-numbers' ),
					),
					'default'           => 'next_batch',
					'css'               => 'min-width: 400px;',
					'custom_attributes' => array(
						'data-show_for_option_name' => $this->get_option_key() . '[use_po_prices]',
					),
				),
				'limit_cart_qty_to_keep_po_price' => array(
					'title'             => __( 'Limit cart quantity to keep price', 'woocommerce-product-batch-numbers' ),
					'type'              => 'checkbox',
					'description'       => __( 'Will limit the cart quantity to the next batch available stock to keep the PO price. This limit is applicable only to products with stock management enabled.', 'woocommerce-product-batch-numbers' ),
					'custom_attributes' => array(
						'data-show_for_option_name' => $this->get_option_key() . '[use_po_prices]',
					),
				),
			);
		}

		/**
		 * Returns an array settings for the display tab.
		 *
		 * @return array
		 */
		private function display_form_fields() {
			return array(
				'section_display_batch_data'                      => array(
					'title'   => '',
					'section' => 'display',
					'type'    => 'title',
					'id'      => 'title',
				),
				'display_batch_data_in_order_admin'               => array(
					'title'       => __( 'Order admin (backend)', 'woocommerce-product-batch-numbers' ),
					'type'        => 'select',
					'options'     => array(
						'no'          => __( 'No', 'woocommerce-product-batch-numbers' ),
						'number'      => __( 'Batch number only', 'woocommerce-product-batch-numbers' ),
						'date'        => __( 'Expiration date only', 'woocommerce-product-batch-numbers' ),
						'number+date' => __( 'Batch number + expiration date', 'woocommerce-product-batch-numbers' ),
					),
					'default'     => 'no',
					'css'         => 'min-width: 400px;',
					'description' => __( 'Display items batch data in the order admin page.', 'woocommerce-product-batch-numbers' ),
				),
				'display_batch_data_in_cart'                      => array(
					'title'       => __( 'Cart', 'woocommerce-product-batch-numbers' ),
					'type'        => 'select',
					'options'     => array(
						'no'          => __( 'No', 'woocommerce-product-batch-numbers' ),
						'number'      => __( 'Batch number only', 'woocommerce-product-batch-numbers' ),
						'date'        => __( 'Expiration date only', 'woocommerce-product-batch-numbers' ),
						'number+date' => __( 'Batch number + expiration date', 'woocommerce-product-batch-numbers' ),
					),
					'default'     => 'no',
					'css'         => 'min-width: 400px;',
					'description' => __( 'Preview the items batch consumption in the cart.', 'woocommerce-product-batch-numbers' ),
				),
				'display_batch_data_in_product_pages'             => array(
					'title'       => __( 'Product pages', 'woocommerce-product-batch-numbers' ),
					'type'        => 'select',
					'options'     => array(
						'no'          => __( 'No', 'woocommerce-product-batch-numbers' ),
						'number'      => __( 'Batch number only', 'woocommerce-product-batch-numbers' ),
						'date'        => __( 'Expiration date only', 'woocommerce-product-batch-numbers' ),
						'number+date' => __( 'Batch number + expiration date', 'woocommerce-product-batch-numbers' ),
					),
					'default'     => 'no',
					'css'         => 'min-width: 400px;',
					'description' => __( 'Display batch data in the product page.', 'woocommerce-product-batch-numbers' ),
				),
				'display_batch_data_in_product_pages_location'    => array(
					'title'             => __( 'Location', 'woocommerce-product-batch-numbers' ),
					'type'              => 'select',
					'options'           => $this->wc_frontend_product_page_hooks(),
					'css'               => 'min-width: 400px;',
					'secondary'         => true,
					'custom_attributes' => array(
						'data-show_for_option_name'   => $this->get_option_key() . '[display_batch_data_in_product_pages]',
						'data-show_for_option_values' => json_encode( array( 'number', 'date', 'number+date' ) ),
					),
				),
				'display_batch_data_in_product_pages_limit'       => array(
					'title'             => __( 'Limit results', 'woocommerce-product-batch-numbers' ),
					'type'              => 'number',
					'description'       => __( 'Limit the number of batch numbers to display.', 'woocommerce-product-batch-numbers' ),
					'secondary'         => true,
					'size'              => 1,
					'css'               => 'min-width: 60px;',
					'custom_attributes' => array(
						'data-show_for_option_name'   => $this->get_option_key() . '[display_batch_data_in_product_pages]',
						'data-show_for_option_values' => json_encode( array( 'number', 'date', 'number+date' ) ),
					),
				),
				'display_batch_data_in_product_pages_time_window' => array(
					'title'             => __( 'Window of time', 'woocommerce-product-batch-numbers' ),
					'type'              => 'display_batch_data_in_product_pages_time_window',
					'secondary'         => true,
					'css'               => 'min-width: 60px;',
					'custom_attributes' => array(
						'data-show_for_option_name'   => $this->get_option_key() . '[display_batch_data_in_product_pages]',
						'data-show_for_option_values' => json_encode( array( 'number', 'date', 'number+date' ) ),
					),
				),
				'display_batch_data_in_formatted_item_meta'       => array(
					'title'       => __( 'Formatted item meta', 'woocommerce-product-batch-numbers' ),
					'type'        => 'select',
					'options'     => array(
						'no'          => __( 'No', 'woocommerce-product-batch-numbers' ),
						'number'      => __( 'Batch number only', 'woocommerce-product-batch-numbers' ),
						'date'        => __( 'Expiration date only', 'woocommerce-product-batch-numbers' ),
						'number+date' => __( 'Batch number + expiration date', 'woocommerce-product-batch-numbers' ),
					),
					'default'     => 'no',
					'css'         => 'min-width: 400px;',
					'description' => __( 'Include the batch data in the formatted order item meta data. This will display the number under the product name in order emails and PDF documents.', 'woocommerce-product-batch-numbers' ),
				),
				'display_batch_column_products'                   => array(
					'title'       => __( 'Product list (backend)', 'woocommerce-product-batch-numbers' ),
					'type'        => 'select',
					'options'     => array(
						'no'          => __( 'No', 'woocommerce-product-batch-numbers' ),
						'number'      => __( 'Batch number only', 'woocommerce-product-batch-numbers' ),
						'date'        => __( 'Expiration date only', 'woocommerce-product-batch-numbers' ),
						'number+date' => __( 'Batch number + expiration date', 'woocommerce-product-batch-numbers' ),
					),
					'default'     => 'no',
					'css'         => 'min-width: 400px;',
					'description' => __( 'Display a column with batch data in the product list.', 'woocommerce-product-batch-numbers' ),
				),
				'display_batch_column_orders'                     => array(
					'title'       => __( 'Order list (backend)', 'woocommerce-product-batch-numbers' ),
					'type'        => 'select',
					'options'     => array(
						'no'          => __( 'No', 'woocommerce-product-batch-numbers' ),
						'number'      => __( 'Batch number only', 'woocommerce-product-batch-numbers' ),
						'date'        => __( 'Expiration date only', 'woocommerce-product-batch-numbers' ),
						'number+date' => __( 'Batch number + expiration date', 'woocommerce-product-batch-numbers' ),
					),
					'default'     => 'no',
					'css'         => 'min-width: 400px;',
					'description' => __( 'Display a column with batch data in the order list.', 'woocommerce-product-batch-numbers' ),
				),
			);
		}

		/**
		 * Returns an array settings for the email tab.
		 *
		 * @return array
		 */
		private function email_form_fields() {
			return array(
				'section_email'                            => array(
					'title'   => '',
					'section' => 'email',
					'type'    => 'title',
					'id'      => 'title',
				),
				'expiration_offset_notification'           => array(
					'title'       => __( 'On offset', 'woocommerce-product-batch-numbers' ),
					'type'        => 'checkbox',
					'description' => __( 'Send notification email upon reaching offset.', 'woocommerce-product-batch-numbers' ),
				),
				'expiration_offset_notification_every_day' => array(
					'title'       => __( 'Every day until expiration', 'woocommerce-product-batch-numbers' ),
					'type'        => 'checkbox',
					'description' => __( 'Notify every day, by email, until the batch expires.', 'woocommerce-product-batch-numbers' ),
				),
				'sold_out_notification'                    => array(
					'title'       => __( 'On sold out', 'woocommerce-product-batch-numbers' ),
					'type'        => 'checkbox',
					'description' => __( 'Automatically send email notifications when a batch is sold out (no stock available).', 'woocommerce-product-batch-numbers' ),
				),
				'expiration_notification'                  => array(
					'title'       => __( 'On expiration', 'woocommerce-product-batch-numbers' ),
					'type'        => 'checkbox',
					'description' => __( 'Automatically send email notifications on batch expiration.', 'woocommerce-product-batch-numbers' ),
				),
				'archived_on_offset_notification'          => array(
					'title'       => __( 'When archived on offset', 'woocommerce-product-batch-numbers' ),
					'type'        => 'checkbox',
					'description' => __( 'Automatically send email notifications when a batch gets archived on offset.', 'woocommerce-product-batch-numbers' ),
				),
				'expiration_date_offset_notification'      => array(
					'title'       => __( 'On expiration date offset', 'woocommerce-product-batch-numbers' ),
					'type'        => 'checkbox',
					'description' => __( 'Automatically send email notifications on expiration date offset.', 'woocommerce-product-batch-numbers' ),
				),
				'email_address_type'                       => array(
					'title'   => __( 'Email address to receive notifications', 'woocommerce-product-batch-numbers' ),
					'type'    => 'select',
					'options' => array(
						'admin_email'  => __( 'Admin email', 'woocommerce-product-batch-numbers' ),
						'wc_email'     => __( 'WooCommerce email', 'woocommerce-product-batch-numbers' ),
						'custom_email' => __( 'Custom email', 'woocommerce-product-batch-numbers' ),
					),
					'default' => 'admin_email',
					'css'     => 'min-width: 400px;',
				),
				'custom_email_address'                     => array(
					'title'       => __( 'Custom email address', 'woocommerce-product-batch-numbers' ),
					'type'        => 'text',
					'css'         => 'min-width: 400px;',
					'secondary'   => true,
					'description' => __( 'Separate emails by a comma (,).', 'woocommerce-product-batch-numbers' ),
					'custom_attributes' => array(
						'data-show_for_option_name'   => $this->get_option_key() . '[email_address_type]',
						'data-show_for_option_values' => json_encode( array( 'custom_email' ) ),
					),
				),
			);
		}

		/**
		 * Returns an array settings for the email tab.
		 *
		 * @return array
		 */
		private function advanced_form_fields() {
			return array(
				'section_advanced'                 => array(
					'title'   => '',
					'section' => 'advanced',
					'type'    => 'title',
					'id'      => 'title',
				),
				'batch_units'                      => array(
					'title'       => __( 'Batch units', 'woocommerce-product-batch-numbers' ),
					'type'        => 'checkbox',
					'description' => __( 'The default value is 1. Enable this if you want to specify different values per batch, for example if you sell packs. Note that this feature could have unexpected behaviour if the product is managing stock.', 'woocommerce-product-batch-numbers' ),
				),
				'delete_duplicate_shared_products' => array(
					'title'       => __( 'Delete duplicate shared products', 'woocommerce-product-batch-numbers' ),
					'type'        => 'checkbox',
					'description' => __( 'Only enable this if you are having issues with duplicated products in batches. A job will run once day at midnight to delete duplicates.', 'woocommerce-product-batch-numbers' ),
				),
				'log_batch_deletion'               => array(
					'title'       => __( 'Register the batch deletion', 'woocommerce-product-batch-numbers' ),
					'type'        => 'checkbox',
					'description' => __( 'Registers batch deletions in a log file.', 'woocommerce-product-batch-numbers' ),
				),
			);
		}

		/**
		 * @return string
		 */
		private function get_active_tab() {
			$active_section = isset( $_REQUEST['section'] ) ? sanitize_text_field( $_REQUEST['section'] ) : '';

			// When the request is coming from form submission.
			if ( empty ( $active_section ) && ! empty( $_REQUEST ) && isset( $_REQUEST['_wp_http_referer'] ) ) {
				$query_string = parse_url( $_REQUEST['_wp_http_referer'], PHP_URL_QUERY );
				parse_str( $query_string, $query_params );
				$active_section = isset( $query_params['section'] ) ? sanitize_text_field( $query_params['section'] ) : '';
			}

			return empty( $active_section ) ? 'general' : $active_section ;
		}

	}

}