<?php

namespace WPO\WC\Product_Batch_Numbers\Settings;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if ( ! class_exists( 'wcpbnSettingsOverview' ) ) {

	class wcpbnSettingsOverview extends wcpbnAbstractSettings {

		protected static $_instance = null;

		/**
		 * Instance
		 */
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		/**
		 * Construct.
		 */
		public function __construct() {
			$this->id = 'overview';

			add_action( 'wpo_wcpbn_settings_tab_content', array( $this, 'tab_content' ), 10, 2 );

			// screen options
			add_filter( 'set-screen-option', array( $this, 'set_screen_option' ), 10, 3 );
			add_filter( 'set_screen_option_wpo_wcpbn_batch_numbers_per_page', array( $this, 'set_screen_option' ), 10, 3 );
		}

		/**
		 * Populates the settings page tab with data
		 *
		 * @param  string  $current_tab
		 * @param  string  $current_section
		 * 
		 * @return void
		 */
		public function tab_content( $current_tab, $current_section ) {
			if ( $current_tab != $this->id ) {
				return;
			}
			$post_type = ! empty( $_REQUEST['post_type'] ) ? esc_attr( $_REQUEST['post_type'] ) : 'product';
			?>
			<div class="wpo_wcpbn_settings_tab">
			<?php
				do_action( 'wpo_wcpbn_before_settings_overview', $this->get_option_key() );
				$overview_list_table = include( WPO_WCPBN()->plugin_dir_path . 'src/class-wcpbn-overview-list-table.php' );
				$overview_list_table->prepare_items();
				// Display status filters
				$overview_list_table->views();
				
				$search_by = apply_filters( 'wpo_wcpbn_overview_list_search_by_options', [
					'batch'       	=> __( 'Batch', 'woocommerce-product-batch-numbers' ),
					'supplier'    	=> __( 'Supplier', 'woocommerce-product-batch-numbers' ),
					'id'          	=> __( 'ID', 'woocommerce-product-batch-numbers' ),
					'location'    	=> __( 'Location', 'woocommerce-product-batch-numbers' ),
					'product_sku' 	=> __( 'Product SKU', 'woocommerce-product-batch-numbers' ),
					'product_id'  	=> __( 'Product ID', 'woocommerce-product-batch-numbers' ),
					'product_title' => __( 'Product Name', 'woocommerce-product-batch-numbers' ),
					'product_meta'  => __( 'Product Metadata (key::value)', 'woocommerce-product-batch-numbers' ),
				] );

				$search_by_filter = ( isset( $_GET['search_by'] ) && ! empty( $_GET['search_by'] ) ) ? esc_attr( $_GET['search_by'] ) : '';
				?>
				<!-- Display Search Form -->
				<form id="wpo-wcpbn-search" method="get">
					<input type="hidden" name="post_type" value="<?= $post_type; ?>">
					<input type="hidden" name="page" value="<?= esc_attr( $_REQUEST['page'] ); ?>">
					<input type="hidden" name="tab" value="overview">
					<fieldset>
						<?php $overview_list_table->search_box( __( 'Search', 'woocommerce-product-batch-numbers' ), 'wcpbn' ); ?>
						<span style="float:right; margin-right:5px;">
							<label for="search_by"><?php _e( 'Search by', 'woocommerce-product-batch-numbers' ); ?></label>
							<select id="search_by" name="search_by">
								<?php
									foreach ( $search_by as $key => $name ) {
										?>
										<option value="<?= $key; ?>" <?php echo selected( $search_by_filter, $key, false ); ?> ><?= $name; ?></option>
										<?php
									}
								?>
							</select>
						</span>
					</fieldset>
				</form>
				<!-- Display Table -->
				<form id="wpo-wcpbn-overview" method="post">
					<input type="hidden" name="page" value="<?php echo $_REQUEST['page'] ?>" />
					<?php $overview_list_table->display(); ?>
				</form>
				<?php
				do_action( 'wpo_wcpbn_after_settings_overview', $this->get_option_key() );
			?>
			</div>
			<?php
		}

		/**
		 * Adds screen options.
		 *
		 * @return void
		 */
		public function screen_options() {
			if ( get_current_screen()->id != 'product_page_wpo-batch-numbers' ) {
				return;
			}

			$active_tab = isset( $_GET[ 'tab' ] ) ? $_GET[ 'tab' ] : 'overview';
			if ( $active_tab == 'overview' ) {
				add_screen_option(
					'per_page',
					array(
						'label'   => __( 'Batch numbers per page', 'woocommerce-product-batch-numbers' ),
						'default' => 10,
						'option'  => 'wpo_wcpbn_batch_numbers_per_page'
					)
				);
			}
	
		}

		/**
		 * Set screen options.
		 *
		 * @return mixed
		 */
		public function set_screen_option( $screen_option, $option, $value ) {
			if ( $option == 'wpo_wcpbn_batch_numbers_per_page' ) {
				$per_page = absint( $value );
				if ( $per_page > 100 ) {
					$per_page = 100;
				}
				return $per_page;
			}
			return $screen_option;
		}

	}

}