<?php

namespace WPO\WC\Product_Batch_Numbers\Settings;

use WPO\WC\Product_Batch_Numbers\Settings\wcpbnAbstractSettings;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if ( ! class_exists( 'wcpbnSettingsRest' ) ) {

	class wcpbnSettingsRest extends wcpbnAbstractSettings {

		protected static $_instance = null;

		/**
		 * Instance
		 */
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		/**
		 * Construct.
		 */
		public function __construct() {
			$this->id = 'rest';

			add_action( 'wpo_wcpbn_settings_tab_content', array( $this, 'tab_content' ), 10, 2 );
		}

		/**
		 * Populates the settings page tab with data
		 *
		 * @param  string  $current_tab
		 * @param  string  $current_section
		 * 
		 * @return void
		 */
		public function tab_content( $current_tab, $current_section ) {
			if ( $current_tab != $this->id ) {
				return;
			}
			?>
			<div id="rest-instructions">
				<p><?php _e( 'This plugin takes advantage of the WooCommerce REST API to provide more flexibility for those who use remote requests.', 'woocommerce-product-batch-numbers' ); ?></p>
				<p>
					<?php
						printf(
							/* translators: <a></a> tags */
							__( 'You can enable the REST API %1$shere%2$s.', 'woocommerce-product-batch-numbers' ),
							'<a href="' . esc_url( admin_url( 'admin.php?page=wc-settings&tab=advanced&section=keys' ) ) . '">',
							'</a>'
						);
					?>
				</p>
				<div>
					<h3><?php _e( 'Options', 'woocommerce-product-batch-numbers' ); ?></h3>
					<table class="widefat">
						<thead>
							<tr>
								<td><?php _e( 'Data key', 'woocommerce-product-batch-numbers' ); ?></td>
								<td><?php _e( 'Example', 'woocommerce-product-batch-numbers' ); ?></td>
								<td><?php _e( 'Default value', 'woocommerce-product-batch-numbers' ); ?></td>
								<td><?php _e( 'Description', 'woocommerce-product-batch-numbers' ); ?></td>
							</tr>
						</thead>
						<tbody>
								<?php
									$data = array(
										'id' => array(
											'description' => __( 'Required if you want to update or delete a certain batch.', 'woocommerce-product-batch-numbers' ),
											'example'     => '"id": "23"',
											'default'     => '',
										),
										'delete' => array(
											'description' => __( 'Required if you want to delete a certain batch.', 'woocommerce-product-batch-numbers' ),
											'example'     => '"delete": "yes"',
											'default'     => '',
										),
										'force_delete' => array(
											'description' => __( 'Required if you want to delete a certain batch and ignore the shared products restriction. You don\'t need to include <code>delete</code> using this.', 'woocommerce-product-batch-numbers' ),
											'example'     => '"force_delete": "yes"',
											'default'     => '',
										),
										'date_expiry' => array(
											'description' => __( 'Required if you want to update or set a new expiry date.', 'woocommerce-product-batch-numbers' ),
											'example'     => '"date_expiry": "2023-12-31 00:00:00"',
											'default'     => '',
										),
										'user_id' => array(
											'description' => __( 'Required if you want to update or set a new user ID.', 'woocommerce-product-batch-numbers' ),
											'example'     => '"user_id": "1"',
											'default'     => '<code>1</code>',
										),
										'batch_number' => array(
											'description' => __( 'Required if you want to update or set a new number.', 'woocommerce-product-batch-numbers' ),
											'example'     => '"batch_number": "1234567890"',
											'default'     => '',
										),
										'supplier' => array(
											'description' => __( 'Required if you want to update or set a new supplier.', 'woocommerce-product-batch-numbers' ),
											'example'     => '"supplier": "WP Overnight"',
											'default'     => '',
										),
										'location' => array(
											'description' => __( 'Required if you want to update or set a new stock location.', 'woocommerce-product-batch-numbers' ),
											'example'     => '"location": "ABC123"',
											'default'     => '',
										),
										'quantity' => array(
											'description' => __( 'Required if you want to set a new quantity.', 'woocommerce-product-batch-numbers' ),
											'example'     => '"quantity": "20"',
											'default'     => '',
										),
										'quantity_available' => array(
											'description' => __( 'Required if you want to update the available stock quantity.', 'woocommerce-product-batch-numbers' ),
											'example'     => '"quantity_available": "10"',
											'default'     => '',
										),
										'status' => array(
											'description' => __( 'Required if you want to update or set a new status. Valid: ', 'woocommerce-product-batch-numbers' ) . ' <code>active</code>, <code>archived</code>',
											'example'     => '"status": "archived"',
											'default'     => '<code>active</code>',
										),
									);
									
									foreach ( $data as $key => $row ) {
										?>
										<tr>
											<td><code><?php echo $key; ?></code></td>
											<td><code class="language-json"><?php echo $row['example']; ?></code></td>
											<td><?php echo $row['default']; ?></td>
											<td><?php echo $row['description']; ?></td>
										</tr>
										<?php
									}
								?>
						</tbody>
					</table>
				</div>
				<div>
					<h3><?php _e( 'Endpoint', 'woocommerce-product-batch-numbers' ); ?></h3>
					<table class="widefat">
						<thead>
							<tr>
								<td><?php _e( 'Methods', 'woocommerce-product-batch-numbers' ); ?></td>
								<td><?php _e( 'URL', 'woocommerce-product-batch-numbers' ); ?></td>
							</tr>
						</thead>
						<tbody>
							<tr>
								<td><code>GET</code>, <code>PUT</code></td>
								<td><code><?php echo esc_url( get_site_url() . '/wp-json/wc/v3/products/' ); ?><strong>{product_id}</strong></code></td>
							</tr>
						</tbody>
					</table>
				</div>
				<div>
					<h3><?php _e( 'Examples', 'woocommerce-product-batch-numbers' ); ?></h3>
					<div class="snippet">
						<label><?php _e( 'Insert a new batch', 'woocommerce-product-batch-numbers' ); ?></label>
						<p><?php _e( 'The <code>id</code> should be omitted because it will be created automatically.', 'woocommerce-product-batch-numbers' ); ?></p>
						<pre><code class="language-json">{
  "batches": [
    {
      "batch_number": "EOEOEOEOEOEOE",
      "date_expiry": "2023-12-31 00:00:00",
      "quantity": "50"
    }
  ]
}</code></pre>
					</div>
					<div class="snippet">
						<label><?php _e( 'Delete a batch', 'woocommerce-product-batch-numbers' ); ?></label>
						<p><?php _e( 'The <code>id</code> should be included to identify the batch to be deleted.', 'woocommerce-product-batch-numbers' ); ?></p>
						<pre><code class="language-json">{
  "batches": [
    {
      "id": "53",
      "delete": "yes"
    }
  ]
}</code></pre>
					</div>
					<div class="snippet">
						<label><?php _e( 'Update a batch', 'woocommerce-product-batch-numbers' ); ?></label>
						<p><?php _e( 'The <code>id</code> should be included to identify the batch to be updated. In this case we are updating the available stock quantity to 20.', 'woocommerce-product-batch-numbers' ); ?></p>
						<pre><code class="language-json">{
  "batches": [
    {
      "id": "53",
      "quantity_available": "20"
    }
  ]
}</code></pre>
					</div>
				</div>
			</div>
			<?php
		}

	}

}
