<?php

namespace WPO\WC\Product_Batch_Numbers\Settings;

use WPO\WC\Product_Batch_Numbers\wcpbnBatchNumberStatus;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if ( ! class_exists( 'wcpbnSettingsTools' ) ) {

	class wcpbnSettingsTools extends wcpbnAbstractSettings {

		protected static $_instance = null;

		/**
		 * Instance
		 */
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		/**
		 * Construct
		 */
		public function __construct() {
			$this->id = 'tools';

			add_action( 'wpo_wcpbn_settings_tab_content', array( $this, 'tab_content' ), 10, 2 );
			add_filter( 'woocommerce_json_search_found_products', array( $this, 'return_included_products' ) );

			// ajax
			add_action( 'wp_ajax_wpo_wcpbn_batch_tools', array( $this, 'ajax_tools' ) );
		}

		/**
		 * Populate the settings page tab with data
		 *
		 * @param  string  $current_tab
		 * @param  string  $current_section
		 * 
		 * @return void
		 */
		public function tab_content( $current_tab, $current_section ) {
			if ( $current_tab != $this->id ) {
				return;
			}
			?>
			<div class="wpo_wcpbn_settings_tab">
			<?php
				do_action( 'wpo_wcpbn_before_settings_tools', $this->get_option_key() );
				
				?>
				<div class="wpo_wcpbn_batch_tools_search">
					<input type="search" id="batch_id_search_input" name="batch_id_search_input" value="<?php echo isset( $_REQUEST['batch'] ) ? esc_attr( $_REQUEST['batch'] ) : ''; ?>">
					<a href="#" class="button button-primary batch-search-button"><?php _e( 'Search batch', 'woocommerce-product-batch-numbers' ); ?></a>
				</div>
				<?php
				
				if ( isset( $_REQUEST['batch'] ) && ! empty( $_REQUEST['batch'] ) ) {
					$batch_id = esc_attr( $_REQUEST['batch'] );
					$batch    = wpo_wcpbn_get_batches( array( 'id' => $batch_id ) );
					
					if ( empty( $batch ) ) {
						echo '<div class="notice notice-warning inline"><p>' . __( 'Batch not found!', 'woocommerce-product-batch-numbers' ) . '</p></div>';
						return;
					}
					
					$batch            = is_array( $batch ) ? reset( $batch ) : $batch;
					$available_status = wcpbnBatchNumberStatus::batch_number_status();
					
					if ( '0000-00-00 00:00:00' !== $batch->date_created && '1000-01-01 00:00:00' !== $batch->date_created ) {
						$batch_date_created = strtotime( $batch->date_created );
					} else {
						$batch_date_created = '';
					}
					
					if ( '0000-00-00 00:00:00' !== $batch->date_expiry && '1000-01-01 00:00:00' !== $batch->date_expiry ) {
						$batch_date_expiry = strtotime( $batch->date_expiry );
					} else {
						$batch_date_expiry = '';
					}
					
					$all_shared_ids = wpo_wcpbn_get_batch_shared_product_ids( $batch_id );
					
					// pass stock locations setting status
					$stock_locations_enabled = wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'stock_locations' ) );
					
					// pass batch units
					$batch_units_enabled = wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'batch_units' ) );
					
					include_once( WPO_WCPBN_TEMPLATES_PATH . '/template-wcpbn-batch-tools.php' );
					
				} else {
					echo '<p>' . __( 'Please introduce a batch ID to see the tools.', 'woocommerce-product-batch-numbers' ) . '</p>';
				}
				
				do_action( 'wpo_wcpbn_after_settings_tools', $this->get_option_key() );
			?>
			</div>
			<?php
		}

		/**
		 * Returns only the products that match the original request
		 * 
		 * @param  array $products from the initial search
		 * 
		 * @return array
		 */
		public function return_included_products( $products ) {
			$new_products = $products;

			// Make sure it's being called from the proper place
			$is_batch_tools_page = false;
			$referer             = parse_url( wp_get_referer() );
			
			if ( isset( $referer['query'] ) ) {
				parse_str( $referer['query'], $params );

				if (
					isset( $params['page'] ) &&
					'wpo-batch-numbers' === $params['page'] &&
					isset( $params['tab'] ) &&
					'tools' === $params['tab']
				) {
					$is_batch_tools_page = true;
				}
			}

			if (
				$is_batch_tools_page &&
				isset( $_REQUEST['include'] ) &&
				is_array( $_REQUEST['include'] )
			) {
				$include      = array_flip( $_REQUEST['include'] );
				$new_products = array_intersect_key( $products, $include );
			}

			return $new_products;
		}
		
		public function ajax_tools() {
			check_ajax_referer( 'wpo_wcpbn', 'nonce' );
			
			$request     = stripslashes_deep( $_REQUEST );
			$tools_forms = array( 'batch-edit', 'batch-add-product', 'batch-remove-product', 'batch-add', 'batch-edit-po', 'batch-add-po', 'batch-remove-po' );
			
			if ( empty( $request['data'] ) || empty( $request['form'] ) || ! in_array( $request['form'], $tools_forms ) || empty( $request['action'] ) || $request['action'] != 'wpo_wcpbn_batch_tools' ) {
				return;
			}
			
			// parse form data
			parse_str( $request['data'], $form_data );
			$form     = esc_attr( $request['form'] );
			$query    = false;
			$messages = array(
				// default
				'error' => __( 'An error occurred!', 'woocommerce-product-batch-numbers' ),
			);
			
			switch ( $form ) {
				case 'batch-edit':
					$data  = array();
					$where = array();
					
					foreach ( $form_data as $key => $value ) {
						$column = str_replace( "{$form}-", '', $key );
						if ( $column == 'quantity_available' ) {
							$value = wpo_wcpbn_force_stock_amount_type( $value );
						} elseif ( $column == 'id' ) {
							$where[$column] = intval( $value );
						} else {
							$value = sanitize_text_field( $value );
						}
						$data[$column] = $value;
					}
					
					if ( ! empty( $data ) && ! empty( $where ) ) {
						$query = wpo_wcpbn_update_batch( $where, $data );
					}
					
					$messages = array(
						'success' => __( 'Batch data saved successfully!', 'woocommerce-product-batch-numbers' ),
						'error'   => __( 'An error occurred while saving the batch data.', 'woocommerce-product-batch-numbers' ),
					);
					break;
				case 'batch-add-product':
					$batch_id = ! empty( $form_data["{$form}-id"] ) ? intval( $form_data["{$form}-id"] ) : false;
					if ( $batch_id ) {
						$batch        = wpo_wcpbn_get_batches( array( 'id' => $batch_id ) );
						if ( ! empty( $batch ) ) {
							$batch        = is_array( $batch ) ? reset( $batch ) : $batch;
							$selected_ids = ! empty( $form_data["{$form}-selected_ids"] ) && is_array( $form_data["{$form}-selected_ids"] ) ? $form_data["{$form}-selected_ids"] : [];
							$query        = wpo_wcpbn_add_products_to_batch( $batch->id, $selected_ids );
						}
						$messages     = array(
							'success' => __( 'Product(s) added successfully!', 'woocommerce-product-batch-numbers' ),
							'error'   => __( 'An error occurred while adding the product(s).', 'woocommerce-product-batch-numbers' ),
						);
					}
					break;
				case 'batch-remove-product':
					$batch_id     = ! empty( $form_data["{$form}-id"] ) ? intval( $form_data["{$form}-id"] ) : false;
					if ( $batch_id ) {
						$batch = wpo_wcpbn_get_batches( array( 'id' => $batch_id ) );
						if ( ! empty( $batch ) ) {
							$batch        = is_array( $batch ) ? reset( $batch ) : $batch;
							$selected_ids = ! empty( $form_data["{$form}-selected_ids"] ) && is_array( $form_data["{$form}-selected_ids"] ) ? $form_data["{$form}-selected_ids"] : [];
							$query        = wpo_wcpbn_remove_products_from_batch( $batch->id, $selected_ids );
						}
						$messages     = [
							'success' => __( 'Product(s) removed successfully!', 'woocommerce-product-batch-numbers' ),
							'error'   => __( 'An error occurred while removing the product(s).', 'woocommerce-product-batch-numbers' ),
						];
					}
					break;
				case 'batch-add':
					$data = array();
					
					foreach ( $form_data as $key => $value ) {
						$column = str_replace( "{$form}-", '', $key );
						if ( $column == 'quantity' ) {
							$value = wpo_wcpbn_force_stock_amount_type( $value );
							$data['quantity_available'] = $value;
						} elseif ( $column == 'selected_ids' ) {
							$value  = is_array( $value ) ? $value : array();
							$column = 'shared_ids';
						} else {
							$value = sanitize_text_field( $value );
						}
						$data[$column] = $value;
					}
					
					if ( ! empty( $data ) ) {
						$data['date_created'] = date( 'Y-m-d H:i:s' );
						$query = wpo_wcpbn_insert_batch( $data );
						if ( $query && ! empty( $data['shared_ids'] ) ) {
							foreach ( $data['shared_ids'] as $product_id ) {
								$product = wc_get_product( absint( $product_id ) );
								if ( $product ) {
									wpo_wcpbn_sync_product_stock_with_batches( $product );
								}
							}
						}
					}
					
					$messages = array(
						'success' => __( 'Batch inserted successfully!', 'woocommerce-product-batch-numbers' ),
						'error'   => __( 'An error occurred while inserting the new batch.', 'woocommerce-product-batch-numbers' ),
					);
					break;
				case 'batch-add-po':
					$data = array();
					
					foreach ( $form_data as $key => $value ) {
						$column = str_replace( "{$form}-", '', $key );
						
						if ( false !== strpos( $column, 'cost' ) || false !== strpos( $column, 'price' ) ) {
							$value = wc_format_decimal( floatval( $value ), '' );
						} else {
							$value = sanitize_text_field( $value );
						}
						
						$data[ $column ] = $value;
					}
					
					if ( ! empty( $data ) ) {
						$data['date_created'] = date( 'Y-m-d H:i:s' );
						$query = wpo_wcpbn_insert_batch_purchase_order( $data );
					}
					
					$messages = array(
						'success' => __( 'Purchase order inserted successfully!', 'woocommerce-product-batch-numbers' ),
						'error'   => __( 'An error occurred while inserting the purchase order.', 'woocommerce-product-batch-numbers' ),
					);
					break;
				case 'batch-edit-po':
					$data  = array();
					$id    = null;
					
					foreach ( $form_data as $key => $value ) {
						$column = str_replace( "{$form}-", '', $key );
						
						if ( false !== strpos( $column, 'cost' ) || false !== strpos( $column, 'price' ) ) {
							$value = wc_format_decimal( floatval( $value ), '' );
						} elseif ( 'id' === $column ) {
							$id    = absint( $value );
						} else {
							$value = sanitize_text_field( $value );
						}
						
						$data[ $column ] = $value;
					}
					
					if ( is_numeric( $id ) && 0 < $id && ! empty( $data ) ) {
						$query = wpo_wcpbn_update_batch_purchase_order( $id, $data );
					}
					
					$messages = array(
						'success' => __( 'Purchase order data saved successfully!', 'woocommerce-product-batch-numbers' ),
						'error'   => __( 'An error occurred while saving the purchase order data.', 'woocommerce-product-batch-numbers' ),
					);
					break;
				case 'batch-remove-po':
					$po_id = ! empty( $form_data["{$form}-id"] ) ? absint( $form_data["{$form}-id"] ) : false;
					
					if ( $po_id ) {
						$query    = wpo_wcpbn_delete_batch_purchase_order( absint( $po_id ) );
						$messages = array(
							'success' => __( 'Product(s) removed successfully!', 'woocommerce-product-batch-numbers' ),
							'error'   => __( 'An error occurred while removing the product(s).', 'woocommerce-product-batch-numbers' ),
						);
					}
					break;
			}
			
			if ( $query ) {
				wp_send_json_success( array( 'success' => $messages['success'] ) );
			} else {
				wp_send_json_success( array( 'error' => $messages['error'] ) );
			}
			
			wp_die();
		}

	}

}