<?php

namespace WPO\WC\Product_Batch_Numbers;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if ( ! class_exists( 'wcpbnAdminNotices' ) ) {

	class wcpbnAdminNotices {

		public $option              = 'wpo_wcpbn_notices';
		public $types               = [ 'error', 'warning', 'success', 'info' ];
		public $notices             = [];
		protected static $_instance = null;
		
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}
		
		public function __construct() {
			$this->notices = get_option( $this->option, [] );
			
			add_action( 'admin_notices', [ $this, 'display' ] );
			add_action( 'wp_ajax_wcpbn-notice-dismiss', [ $this, 'ajax_dismiss' ] );
		}
		
		public function save( $message, $type, $dismissible = true ) {
			if ( ! empty( $message ) && in_array( $type, $this->types ) ) {
				$this->notices[] = [
					'type'        => esc_attr( $type ),
					'message'     => wp_kses_post( $message ),
					'dismissible' => $dismissible,
				];
				update_option( $this->option, $this->notices );
			}
		}
		
		public function ajax_dismiss() {
			check_ajax_referer( 'wpo_wcpbn', 'nonce' );
			
			$request = stripslashes_deep( $_REQUEST );
			
			if ( ! isset( $request['key'] ) || ! is_numeric( $request['key'] ) || empty( $request['action'] ) || $request['action'] != 'wcpbn-notice-dismiss' ) {
				return;
			}
			
			unset( $this->notices[intval( $request['key'] )] );
			
			$updated = update_option( $this->option, $this->notices );
			
			if ( $updated ) {
				wp_send_json_success();	
			} else {
				wp_send_json_error();
			}
			
			wp_die();
		}
		
		public function display() {
			if ( empty( $this->notices ) ) {
				return;
			}
			
			foreach ( $this->notices as $key => $notice ) {
				$dismissible = $notice['dismissible'] ? 'is-dismissible' : '';
				?>
				<div class="notice notice-<?= $notice['type']; ?> <?= $dismissible; ?> wcpbn-notice" data-key="<?= $key; ?>"><p><?= $notice['message']; ?></p></div>
				<?php
			}
		}

	}

}