<?php
/**
 * Batches Class
 *
 * @since 1.2.4
 */

namespace WPO\WC\Product_Batch_Numbers;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if ( ! class_exists( 'wcpbnBatches' ) ) {

	class wcpbnBatches {

		protected        $email_address;
		protected static $_instance      = null;

		/**
		 * Instance
		 */
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		/**
		 * Construct
		 */
		public function __construct() {
			$this->email_address = $this->get_email_address_for_notifications();

			// schedule actions at midnight
			add_action( 'admin_init', array( $this, 'schedule_actions_midnight' ) );
			
			// add batch form
			add_action( 'wpo_wcpbn_before_settings_overview', array( $this, 'batch_add_new_form' ), 1 );

			// archive batches
			if ( ! empty( WPO_WCPBN()->classes['settings']->options->get_option( 'archive_batches' ) ) && WPO_WCPBN()->classes['settings']->options->get_option( 'archive_batches' ) != 'no' ) {
				add_action( 'wcpbn_schedule_archive_batches', array( $this, 'archive_batches' ) );
			}

			// expiry offset notification
			if ( wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'expiration_offset_notification' ) ) ) {
				add_action( 'wcpbn_schedule_expiry_offset_notification', array( $this, 'expiry_offset_notification' ) );
				add_action( 'wcpbn_schedule_expiry_offset', array( $this, 'expiry_offset_notification' ) ); // legacy hook
			}

			// stock email notification
			if ( wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'sold_out_notification' ) ) ) {
				add_action( 'wcpbn_schedule_sold_out_notification', array( $this, 'sold_out_notification' ) );
				add_action( 'wcpbn_schedule_stock_email_notification', array( $this, 'sold_out_notification' ) ); // legacy hook
			}

			// batch expired email notification
			if ( wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'expiration_notification' ) ) ) {
				add_action( 'wcpbn_schedule_expiry_notification', array( $this, 'expiry_notification' ) );                     // action that runs everyday looking for database active batches that could be archived
				add_action( 'wcpbn_schedule_batch_expired_notification', array( $this, 'batch_expired_notification' ) );       // single batch archive
				add_action( 'wcpbn_schedule_batch_expired_email_notification', array( $this, 'batch_expired_notification' ) ); // legacy hook
			}

			// batch expiring offset email notification
			if ( wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'expiration_date_offset_notification' ) ) || wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'expiration_offset_notification' ) ) ) {
				add_action( 'wcpbn_schedule_batch_expiration_date_offset_notification', array( $this, 'batch_expiration_date_offset_email_notification' ) );
			}

			// batch archived on offset email notification
			if ( wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'archived_on_offset_notification' ) ) ) {
				add_action( 'wcpbn_schedule_batch_archived_on_offset_notification', array( $this, 'batch_archived_on_offset_notification' ) );
			}
		}
		
		public function batch_add_new_form( $option_key ) {
			if ( $option_key != 'wpo_wcpbn_settings_overview' ) {
				return;
			}
			
			// pass stock locations setting status
			$stock_locations_enabled = wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'stock_locations' ) );
			
			// pass batch units
			$batch_units_enabled = wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'batch_units' ) );
						
			include_once( WPO_WCPBN_TEMPLATES_PATH.'/template-wcpbn-add-batch.php' );
		}

		/**
		 * Retrieves the correct email address for notifications
		 *
		 * @return string
		 */
		public function get_email_address_for_notifications() {
			$email_type = WPO_WCPBN()->classes['settings']->options->get_option( 'email_address_type' );
			switch ( $email_type ) {
				case 'custom_email':
					$emails_array = explode( ',', WPO_WCPBN()->classes['settings']->options->get_option( 'custom_email_address' ) );
					$valid_emails = array_map( function ( $email ) {
						return filter_var( trim( $email ), FILTER_VALIDATE_EMAIL );
					}, $emails_array );

					$output_email = implode( ',', $valid_emails );
					break;
				case 'wc_email':
					$output_email = get_option( 'woocommerce_stock_email_recipient' );
					break;
				case 'admin_email':
				default:
					$output_email = get_option( 'admin_email' );
					break;
			}

			if ( empty( $output_email ) ) {
				$output_email = get_option( 'admin_email' );
			}

			return $output_email;
		}

		/**
		 * Archive batches with Action Scheduler at midnight
		 *
		 * @return void
		 */
		public function schedule_actions_midnight() {
			if ( ! function_exists( 'as_schedule_recurring_action' ) ) {
				return;
			}

			$timestamp           = apply_filters( 'wpo_wcpbn_as_archive_batches_timestamp', strtotime( 'tomorrow' ) );
			$interval_in_seconds = apply_filters( 'wpo_wcpbn_as_archive_batches_interval_in_seconds', DAY_IN_SECONDS );

			if ( false === as_next_scheduled_action( 'wcpbn_schedule_archive_batches' ) && ! empty( WPO_WCPBN()->classes['settings']->options->get_option( 'archive_batches' ) ) && WPO_WCPBN()->classes['settings']->options->get_option( 'archive_batches' ) != 'no' ) {
				as_schedule_recurring_action( $timestamp, $interval_in_seconds, 'wcpbn_schedule_archive_batches' );
			}
			if ( false === as_next_scheduled_action( 'wcpbn_schedule_expiry_notification' ) && wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'expiration_notification' ) ) ) {
				as_schedule_recurring_action( $timestamp, $interval_in_seconds, 'wcpbn_schedule_expiry_notification' );
			}
			if ( false === as_next_scheduled_action( 'wcpbn_schedule_expiry_offset_notification' ) && wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'expiration_offset_notification' ) ) ) {
				as_schedule_recurring_action( $timestamp, $interval_in_seconds, 'wcpbn_schedule_expiry_offset_notification' );
			}
		}

		/**
		 * Archive batches
		 *
		 * @return void
		 */
		public function archive_batches() {
			$today            = time();
			$updated_products = array();
			$archive_batches  = WPO_WCPBN()->classes['settings']->options->get_option( 'archive_batches' );
			$offset           = wpo_wcpbn_get_global_offset_in_days();

			$query_args = array(
				'date_expiry' => "<{$today}",
				'status'      => 'active',
			);
			$batches = wpo_wcpbn_get_batches( $query_args );

			if ( ! empty( $batches ) ) {
				foreach ( $batches as $batch ) {
					if ( $batch->date_expiry == '0000-00-00 00:00:00' || $batch->date_expiry == '1000-01-01 00:00:00' ) {
						continue;
					}

					$all_shared_ids = wpo_wcpbn_get_batch_shared_product_ids( $batch->id );
					$batch_date     = strtotime( $batch->date_expiry );
					$can_archive    = false;

					switch ( $archive_batches ) {
						case 'expired':
							$can_archive = $today > $batch_date ? true : false;
							break;
						case 'on_offset':
							// product offset
							$products_can_archive = [];
							foreach ( $all_shared_ids as $id ) {
								$offset                 = wpo_wcpbn_get_product_current_offset_in_days( $id );
								$less_x_days            = strtotime( '-'.$offset.' days', $batch_date );
								$products_can_archive[] = ! empty( $less_x_days ) && $less_x_days == $today ? true : false;
							}
							if ( count( array_unique( $products_can_archive ) ) == 1 && reset( $products_can_archive ) === true ) {
								$can_archive = true;
							}
							break;
					}

					// archive
					if ( $can_archive ) {
						$data['status'] = 'archived';
						$output         = wpo_wcpbn_update_batch( $batch->id, $data );

						if ( $output ) {
							// schedule email notification
							if ( $archive_batches == 'expired' && wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'expiration_notification' ) ) ) {
								$this->schedule_batch_expired_notification_email( $batch );
							} elseif ( $archive_batches == 'on_offset' && wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'archived_on_offset_notification' ) ) ) {
								$this->schedule_batch_archived_on_offset_notification_email( $batch );
							}
							
							// update products stock and stock status
							if ( ! empty( $all_shared_ids ) ) {
								foreach ( $all_shared_ids as $pid ) {
									if ( ! empty( $product = wc_get_product( $pid ) ) ) {
										wpo_wcpbn_sync_product_stock_with_batches( $product );
									}
								}
							}
						}
					}
				}
			}
		}

		/**
		 * Expiry date callback
		 *
		 * @return void
		 */
		public function expiry_notification() {
			$today    = time();
			$last_run = get_option( 'wcpbn_expiry_notification_last_action_run_timestamp' );

			// date arg
			if ( ! empty( $last_run ) ) {
				$date_arg = "{$last_run}...{$today}";
			} else {
				$date_arg = "<={$today}";
			}

			$query_args = array(
				'date_expiry' => $date_arg,
				'status'      => 'active',
			);
			$batches = wpo_wcpbn_get_batches( $query_args );

			if ( ! empty( $batches ) ) {
				foreach ( $batches as $batch ) {
					if ( $batch->date_expiry == '0000-00-00 00:00:00' || $batch->date_expiry == '1000-01-01 00:00:00' ) {
						continue;
					}

					$batch_date = strtotime( $batch->date_expiry );
					
					if ( ! empty( $last_run ) ) {
						$expired = ( $batch_date >= $last_run && $today > $batch_date ) ? true : false;
					} else {
						$expired = ( $today > $batch_date ) ? true : false;
					}

					// trigger email
					if ( $expired ) {
						$this->schedule_batch_expired_notification_email( $batch );
						update_option( 'wcpbn_expiry_notification_last_action_run_timestamp', $today );
					}
				}
			}
		}

		/**
		 * Expiry date offset check callback
		 *
		 * @return void
		 */
		public function expiry_offset_notification() {
			$batches = wpo_wcpbn_get_batches( [ 'status' => 'active' ] );

			// compare dates
			if ( ! empty( $batches ) ) {
				foreach ( $batches as $batch ) {
					if ( $batch->date_expiry == '0000-00-00 00:00:00' || $batch->date_expiry == '1000-01-01 00:00:00' ) {
						continue;
					}

					$batch_date       = get_date_from_gmt( $batch->date_expiry, 'Y-m-d' );
					$batch_date       = new \DateTime( $batch_date );
					$current          = new \DateTime( 'now' );
					$diff             = $batch_date->diff( $current )->format( "%a" );
					$offset           = wpo_wcpbn_get_global_offset_in_days();
					$every_day        = wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'expiration_offset_notification_every_day' ) );
					$all_shared_ids   = wpo_wcpbn_get_batch_shared_product_ids( $batch->id );
					
					if ( empty( $all_shared_ids ) ) {
						continue;
					}
					
					foreach ( $all_shared_ids as $id ) {
						$email_sent       = false;
						$notified_batches = get_option( 'wcpbn_date_expiry_global_notification_batches', array() );
						$product          = wc_get_product( $id );
						
						if ( empty( $product ) ) {
							continue;
						}

						try {
							// prevent sending every day if already sent
							if ( ! $every_day && ! empty( $notified_batches ) && in_array( $batch->id, $notified_batches ) ) {
								$email_sent = true;
							}

							// get product offset
							$product_offset = wpo_wcpbn_get_product_current_offset_in_days( $id );
							
							// if product has offset override the global offset
							if ( ! empty( $product_offset ) && is_numeric( $product_offset ) ) {
								$offset = $product_offset;
							}
							
							// trigger email notification
							if ( ! $email_sent && $diff <= $offset ) {
								array_push( $notified_batches, $batch->id );
								update_option( 'wcpbn_date_expiry_global_notification_batches', $notified_batches );
								$this->schedule_batch_expiration_date_offset_email( $batch );
							}

						} catch ( \Throwable $e ) {
							WPO_WCPBN()->classes['logger']->log( "Unable to send email for expired batch ID #{$batch->id}. Error: {$e}", 'critical' );
							continue;
						}
					}
				}
			}
		}

		/**
		 * Schedule batch expiring offset notification emails with AS
		 *
		 * @return void
		 */
		public function schedule_batch_expiration_date_offset_email( $batch ) {
			if ( ! function_exists( 'as_schedule_single_action' ) || empty( $batch ) ) {
				return;
			}

			$timestamp             = apply_filters( 'wpo_wcpbn_as_batch_expiration_date_offset_notification_email_timestamp', strtotime( 'now' ) );
			$scheduled_action_args = array( 'batch' => $batch );

			if ( false === as_next_scheduled_action( 'wcpbn_schedule_batch_expiration_date_offset_notification', $scheduled_action_args ) ) {
				as_schedule_single_action( $timestamp, 'wcpbn_schedule_batch_expiration_date_offset_notification', $scheduled_action_args );
			}
		}

		/**
		 * Process batch expiring offset email notification
		 *
		 * @return void
		 */
		public function batch_expiration_date_offset_email_notification( $batch ) {
			if ( empty( $batch ) ) {
				return;
			}

			$data = $this->get_batch_data_for_emails( $batch );
			if ( empty( $data ) ) {
				return;
			}

			$batch_date       = get_date_from_gmt( $data['date_expiry'], 'Y-m-d' );
			$batch_date       = new \DateTime( $batch_date );
			$current          = new \DateTime( 'now' );
			$days_to_expiry   = intval( $batch_date->diff( $current )->format( "%a" ) );
			$batch_tools_link = '<a href="' . $data['batch_tools'] . '">' . __( 'here', 'woocommerce-product-batch-numbers' ) . '</a>';
			$to               = apply_filters( 'wpo_wcpbn_email_notifications_address', sanitize_text_field( $this->email_address ) ); // legacy filter
			$to               = apply_filters( 'wpo_wcpbn_email_recipients', explode( ',', $to ) ); // array for multiple recipients
			$subject          = __( 'Batch expiring', 'woocommerce-product-batch-numbers' ) . ': ' . $data['batch_number'];

			/* translators: 1. available stock, 2. batch number, 3. days to expiry, 4. batch tools link*/
			$message  = sprintf( __( '%1$s units of Batch no. #%2$s has %3$s days left before expiry. You may want to feature or promote them with an offer to clear the stock. Visit the batch tools page %4$s.', 'woocommerce-product-batch-numbers' ), $data['quantity_available'], $data['batch_number'], $days_to_expiry, $batch_tools_link );
			$message .= $this->get_batch_shared_products_html_for_emails( $data['batch_id'] );
			
			$body     = apply_filters( 'wpo_wcpbn_batch_expiration_date_offset_notification_message', $message, $data );
			$headers  = apply_filters( 'woocommerce_email_headers', 'Content-Type: text/html; charset=UTF-8'."\r\n", 'expiration_date_offset_notification', $data, null );
			$headers  = apply_filters( 'wpo_wcpbn_email_headers', $headers ); // legacy filter
			wc_deprecated_function( 'wpo_wcpbn_email_headers', '2.0.1', 'woocommerce_email_headers' );
			add_filter( 'wp_mail_content_type', function( $content_type ) { return 'text/html'; } ); // forces wp_mail() to use 'text/html'

			$response = wp_mail( $to, $subject, $body, $headers ); // try to send email
		}

		/**
		 * Schedule stock notification emails with AS
		 *
		 * @return void
		 */
		public function schedule_stock_notification_email( $batch ) {
			if ( ! function_exists( 'as_schedule_single_action' ) || empty( $batch ) ) {
				return;
			}

			$timestamp = apply_filters( 'wpo_wcpbn_as_stock_notification_email_timestamp', strtotime( 'now' ) );
			
			if ( false === as_next_scheduled_action( 'wcpbn_schedule_sold_out_notification' ) ) {
				as_schedule_single_action( $timestamp, 'wcpbn_schedule_sold_out_notification', array( 'batch' => $batch ) );
			}
		}

		/**
		 * Process stock email notification
		 *
		 * @return void
		 */
		public function sold_out_notification( $batch ) {
			if ( empty( $batch ) ) {
				return;
			}

			$data = $this->get_batch_data_for_emails( $batch );
			if ( empty( $data ) ) {
				return;
			}

			$batch_tools_link = '<a href="' . $data['batch_tools'] . '">' . __( 'here', 'woocommerce-product-batch-numbers' ) . '</a>';
			$to               = apply_filters( 'wpo_wcpbn_email_notifications_address', sanitize_text_field( $this->email_address ) ); // legacy filter
			$to               = apply_filters( 'wpo_wcpbn_email_recipients', explode( ',', $to ) ); // array for multiple recipients
			$subject          = __( 'Batch end', 'woocommerce-product-batch-numbers' ) . ': ' . $data['batch_number'];

			/* translators: 1. batch number, 2. batch tools link*/
			$message  = sprintf( __( 'Batch no. #%1$s is down to zero stock. Consider adding more %2$s.', 'woocommerce-product-batch-numbers' ), $data['batch_number'], $batch_tools_link );
			$message .= $this->get_batch_shared_products_html_for_emails( $data['batch_id'] );
			
			$body     = apply_filters( 'wpo_wcpbn_stock_notification_message', $message, $data);
			$headers  = apply_filters( 'woocommerce_email_headers', 'Content-Type: text/html; charset=UTF-8'."\r\n", 'stock_notification', $data, null );
			$headers  = apply_filters( 'wpo_wcpbn_email_headers', $headers ); // legacy filter
			wc_deprecated_function( 'wpo_wcpbn_email_headers', '2.0.1', 'woocommerce_email_headers' );
			add_filter( 'wp_mail_content_type', function( $content_type ) { return 'text/html'; } ); // forces wp_mail() to use 'text/html'

			$response = wp_mail( $to, $subject, $body, $headers ); // try to send email
		}

		/**
		 * Schedule batch expired notification emails with AS
		 *
		 * @return void
		 */
		public function schedule_batch_expired_notification_email( $batch ) {
			if ( ! function_exists( 'as_schedule_single_action' ) || empty( $batch ) ) {
				return;
			}

			$timestamp = apply_filters( 'wpo_wcpbn_as_batch_expired_notification_email_timestamp', strtotime( 'now' ) );
			as_schedule_single_action( $timestamp, 'wcpbn_schedule_batch_expired_notification', array( 'batch' => $batch ) );
		}

		/**
		 * Process batch expired email notification
		 *
		 * @return void
		 */
		public function batch_expired_notification( $batch ) {
			if ( empty( $batch ) ) {
				return;
			}

			$data = $this->get_batch_data_for_emails( $batch );
			if ( empty( $data ) ) {
				return;
			}

			$batch_tools_link = '<a hr="' . $data['batch_tools'] . '">' . __( 'here', 'woocommerce-product-batch-numbers' ) . '</a>';
			$to               = apply_filters( 'wpo_wcpbn_email_notifications_address', sanitize_text_field( $this->email_address ) ); // legacy filter
			$to               = apply_filters( 'wpo_wcpbn_email_recipients', explode( ',', $to ) ); // array for multiple recipients
			$subject          = __( 'Batch expired', 'woocommerce-product-batch-numbers' ) . ': ' . $data['batch_number'];

			/* translators: 1. stock available, 2. batch number, 3. batch tools link */
			$message  = sprintf( __( '%1$s units of Batch no. #%2$s have expired. Consider adding more %3$s.', 'woocommerce-product-batch-numbers' ), $data['quantity_available'], $data['batch_number'], $batch_tools_link );
			$message .= $this->get_batch_shared_products_html_for_emails( $data['batch_id'] );
			
			$body     = apply_filters( 'wpo_wcpbn_batch_expired_notification_message', $message, $data );
			$headers  = apply_filters( 'woocommerce_email_headers', 'Content-Type: text/html; charset=UTF-8'."\r\n", 'batch_expired_notification', $data, null );
			$headers  = apply_filters( 'wpo_wcpbn_email_headers', $headers ); // legacy filter
			wc_deprecated_function( 'wpo_wcpbn_email_headers', '2.0.1', 'woocommerce_email_headers' );
			add_filter( 'wp_mail_content_type', function( $content_type ) { return 'text/html'; } ); // forces wp_mail() to use 'text/html'

			$response = wp_mail( $to, $subject, $body, $headers ); // try to send email
		}

		/**
		 * Schedule batch archived on offset notification emails with AS
		 *
		 * @return void
		 */
		public function schedule_batch_archived_on_offset_notification_email( $batch ) {
			if ( ! function_exists( 'as_schedule_single_action' ) || empty( $batch ) ) {
				return;
			}

			$timestamp = apply_filters( 'wpo_wcpbn_as_batch_archived_on_offset_notification_email_timestamp', strtotime( 'now' ) );
			as_schedule_single_action( $timestamp, 'wcpbn_schedule_batch_archived_on_offset_notification', array( 'batch' => $batch ) );
		}

		/**
		 * Process batch archived on offset email notification
		 *
		 * @return void
		 */
		public function batch_archived_on_offset_notification( $batch ) {
			if ( empty( $batch ) ) {
				return;
			}

			$data = $this->get_batch_data_for_emails( $batch );
			if ( empty( $data ) ) {
				return;
			}

			$batch_tools_link = '<a href="' . $data['batch_tools'] . '">' . __( 'here', 'woocommerce-product-batch-numbers' ) . '</a>';
			$to               = apply_filters( 'wpo_wcpbn_email_notifications_address', sanitize_text_field( $this->email_address ) ); // legacy filter
			$to               = apply_filters( 'wpo_wcpbn_email_recipients', explode( ',', $to ) ); // array for multiple recipients
			$subject          = __( 'Batch archived on offset', 'woocommerce-product-batch-numbers' ) . ': ' . $data['batch_number'];

			/* translators: 1. stock available, 2. batch number, 3. batch tools link */
			$message  = sprintf( __( '%1$s units of Batch no. #%2$s have been archived on offset ending. Please consider adding more units %3$s.', 'woocommerce-product-batch-numbers' ), $data['quantity_available'], $data['batch_number'], $batch_tools_link );
			$message .= $this->get_batch_shared_products_html_for_emails( $data['batch_id'] );
			
			$body     = apply_filters( 'wpo_wcpbn_batch_archived_on_offset_notification_message', $message, $data );
			$headers  = apply_filters( 'woocommerce_email_headers', 'Content-Type: text/html; charset=UTF-8'."\r\n", 'batch_archived_on_offset_notification', $data, null );
			$headers  = apply_filters( 'wpo_wcpbn_email_headers', $headers ); // legacy filter
			wc_deprecated_function( 'wpo_wcpbn_email_headers', '2.0.1', 'woocommerce_email_headers' );
			add_filter( 'wp_mail_content_type', function( $content_type ) { return 'text/html'; } ); // forces wp_mail() to use 'text/html'

			$response = wp_mail( $to, $subject, $body, $headers ); // try to send email
		}

		/**
		 * Retrieves the required data to compose the emails
		 *
		 * @return array
		 */
		private function get_batch_data_for_emails( $batch ) {
			$data = array();
			
			if ( empty( $batch ) ) {
				return $data;
			}
			
			if ( is_array( $batch ) && ! isset( $batch[0] ) ) {
				$batch = [ (object) $batch ];
			}
			
			$batch = is_array( $batch ) ? reset( $batch ) : $batch;
			
			$data['batch_number']       = $batch->batch_number;
			$data['batch_id']           = $batch->id;
			$data['quantity_available'] = wpo_wcpbn_force_stock_amount_type( $batch->quantity_available );
			$data['date_expiry']        = $batch->date_expiry;
			$data['batch_tools']        = esc_url_raw( admin_url( 'edit.php?post_type=product&page=wpo-batch-numbers&tab=tools&batch='.$batch->id ) );

			return $data;
		}
		
		private function get_batch_shared_products_html_for_emails( $batch_id ) {
			$shared_product_ids = wpo_wcpbn_get_batch_shared_product_ids( $batch_id );
			$html               = '';
			
			if ( ! empty( $shared_product_ids ) ) {
				$html .= '<p><label>'.__( 'Affected products', 'woocommerce-product-batch-numbers' ) .':</label><ul>';
				foreach ( $shared_product_ids as $product_id ) {
					if ( ! empty( $product = wc_get_product( $product_id ) ) ) {
						$html .= '<li><a href="'.esc_url_raw( admin_url( 'post.php?post='.$product->get_id().'&action=edit' ) ).'">'.$product->get_name().'</a></li>';
					}
				}
				$html .= '</ul></p>';
			}
			
			return $html;
		}

	}

}