<?php

namespace WPO\WC\Product_Batch_Numbers;

use WPO\WC\Product_Batch_Numbers\Database\wcpbnDbBatchNumbers;
use WPO\WC\Product_Batch_Numbers\Database\wcpbnDbPurchaseOrders;
use WPO\WC\Product_Batch_Numbers\Database\wcpbnDbSharedProducts;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if ( ! class_exists( 'wcpbnDatabase' ) ) {

	class wcpbnDatabase {

		/**
		 * @var wcpbnDbBatchNumbers
		 */
		public $batch_numbers;

		/**
		 * @var wcpbnDbSharedProducts
		 */
		public $shared_products;

		/**
		 * @var wcpbnDbPurchaseOrders
		 */
		public $purchase_orders;

		/**
		 * @var array string[]
		 */
		private $allowed_child_classes = array(
			'WPO\WC\Product_Batch_Numbers\Database\wcpbnDbBatchNumbers',
			'WPO\WC\Product_Batch_Numbers\Database\wcpbnDbPurchaseOrders',
			'WPO\WC\Product_Batch_Numbers\Database\wcpbnDbSharedProducts',
		);

		/**
		 * @var array string[]
		 */
		private $allowed_join_callbacks = array( 'join_callback' );

		/**
		 * @var wcpbnDatabase
		 */
		protected static $_instance = null;

		public static function instance(): wcpbnDatabase {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}

			return self::$_instance;
		}

		public function batch_numbers(): wcpbnDbBatchNumbers {
			if ( is_null( $this->batch_numbers ) ) {
				$this->batch_numbers = wcpbnDbBatchNumbers::instance();
			}

			return $this->batch_numbers;
		}

		public function purchase_orders(): wcpbnDbPurchaseOrders {
			if ( is_null( $this->purchase_orders ) ) {
				$this->purchase_orders = wcpbnDbPurchaseOrders::instance();
			}

			return $this->purchase_orders;
		}

		public function shared_products(): wcpbnDbSharedProducts {
			if ( is_null( $this->shared_products ) ) {
				$this->shared_products = wcpbnDbSharedProducts::instance();
			}

			return $this->shared_products;
		}

		/**
		 * @param object|string $class
		 *
		 * @return bool
		 */
		public function is_child_class_allowed( $class ): bool {
			if ( is_object( $class ) ) {
				$class = get_class( $class );
			}

			return in_array( $class, $this->allowed_child_classes, true );
		}

		/**
		 * Validate given callback methods sent to the get_rows() method.
		 *
		 * @param array $join_callback
		 *
		 * @return bool
		 */
		public function is_join_callback_allowed( array $join_callback ): bool {
			return $this->is_child_class_allowed( $join_callback[0] ) && in_array( $join_callback[1], $this->allowed_join_callbacks, true );
		}

	}

}
