<?php

namespace WPO\WC\Product_Batch_Numbers;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if ( ! class_exists( 'wcpbnOrderListing' ) ) {

	class wcpbnOrderListing {

		protected static $_instance = null;

		/**
		 * Instance
		 */
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		public function __construct() {
			if ( ! empty( WPO_WCPBN()->classes['settings']->options->get_option( 'display_batch_column_orders' ) ) && WPO_WCPBN()->classes['settings']->options->get_option( 'display_batch_column_orders' ) != 'no' ) {
				add_filter( 'manage_woocommerce_page_wc-orders_columns', array( $this, 'batch_listing_column' ), 10, 1 ); // WC 7.1+
				add_action( 'manage_woocommerce_page_wc-orders_custom_column', array( $this, 'populate_batch_column' ), 10, 2 ); // WC 7.1+
				add_filter( 'manage_edit-shop_order_columns', array( $this, 'batch_listing_column' ), 10, 1 );
				add_action( 'manage_posts_custom_column', array( $this, 'populate_batch_column' ), 10, 2 );
			}
		}

		public function batch_listing_column( $columns ) {
			$offset          = 6;
			$updated_columns = array_slice( $columns, 0, $offset, true) +
			array( 'order_batch_numbers' => esc_html__( 'Batches', 'woocommerce-product-batch-numbers' ) ) +
			array_slice( $columns, $offset, null, true );
			return $updated_columns;
		}

		public function populate_batch_column( $column_name, $post_or_order_object ) {
			if ( $column_name != 'order_batch_numbers' ) {
				return;
			}
			
			$order = ( $post_or_order_object instanceof \WP_Post ) ? wc_get_order( $post_or_order_object->ID ) : $post_or_order_object;
			if ( ! is_object( $order ) && is_numeric( $order ) ) {
				$order = wc_get_order( absint( $order ) );
			}

			// display order items batch list
			$this->column_batches_list( $order );
		}

		private function column_batches_list( $order ) {
			if ( ! is_object( $order ) ) {
				return;
			}

			foreach ( $order->get_items() as $item_id => $item ) {
				$item_batches = $item->get_meta( '_wcpbn_data' );
				if ( ! empty( $item_batches ) && is_array( $item_batches ) ) {
					// add 'date_expiry' to item batches
					foreach ( $item_batches as $key => $batch ) {
						$batch = wpo_wcpbn_get_batches( [ 'id' => $batch['id'] ] );
						if ( empty( $batch ) ) {
							continue;
						}
						$batch = is_array( $batch ) ? reset( $batch ) : $batch;
						
						if ( isset( $batch->date_expiry ) ) {
							$item_batches[$key]['date_expiry'] = $batch->date_expiry;
						}
					}

					$data_format = WPO_WCPBN()->classes['settings']->options->get_option( 'display_batch_column_orders' );
					$product_id  = ! empty( $item['variation_id'] ) ? $item['variation_id'] : $item['product_id'];
					wpo_wcpbn_print_product_batch_data_formatted( $product_id, $data_format, true, false, '', '', '', $item_batches );
				}
			}
		}

	}

}
