<?php
/**
 * Product Class
 *
 * @since 1.0.0
 */

namespace WPO\WC\Product_Batch_Numbers;

/**
 * If this file is called directly, abort.
 *
 * @since 1.0.0
 */
if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if ( ! class_exists( 'wcpbnProduct' ) ) {

	class wcpbnProduct {

		private          $product_status;
		protected        $orderby;
		protected static $_instance       = null;

		/**
		 * Instance
		 */
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		/**
		 * Construct
		 */
		public function __construct() {
			$this->product_status = wcpbnBatchNumberStatus::batch_number_status();

			// get batch order by priority
			$this->orderby = ( $batch_stock_reduction_priority = WPO_WCPBN()->classes['settings']->options->get_option( 'batch_prioritization' ) ) ? $batch_stock_reduction_priority : 'id';

			add_action( 'woocommerce_product_options_inventory_product_data', array( $this, 'batch_numbers_product_data' ) );
			add_action( 'woocommerce_product_after_variable_attributes', array( $this, 'batch_numbers_product_data_variation' ), 99, 3 );
			add_action( 'woocommerce_process_product_meta', array( $this, 'crud_batch_numbers_product' ), 10, 2 );
			add_action( 'woocommerce_save_product_variation', array( $this, 'crud_batch_numbers_product_variation' ), 10, 2 );
			add_action( 'wp_ajax_new_batch_number_row_html', array( $this, 'new_batch_number_row_html' ) );
			add_action( 'wp_ajax_get_last_added_batch', array( $this, 'get_last_added_batch' ) );
			add_action( 'wp_ajax_crud_batch_numbers_product', array( $this, 'crud_batch_numbers_product' ) );
			add_action( 'wp_ajax_delete_batch_number_row', array( $this, 'delete_batch_number_row' ) );
			add_action( 'wp_ajax_supplier_names', array( $this, 'supplier_names' ) );
			add_action( 'wp_ajax_batch_number_check', array( $this, 'batch_number_check' ) );

			// if WC stock sync setting is enabled
			if ( wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'sync_wc_stock' ) ) ) {
				add_action( 'woocommerce_product_options_stock', array( $this, 'sync_product_simple_stock_button' ), 998 );
				add_action( 'woocommerce_variation_options_pricing', array( $this, 'sync_product_variation_stock_button' ), 998, 3 );
				add_action( 'wp_ajax_sync_product_wc_stock', array( $this, 'sync_product_stock_ajax' ) );
			}

			// delete batches when deleting product
			if ( wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'delete_batch_on_product_delete' ) ) ) {
				add_action( 'before_delete_post', array( $this, 'delete_batches_on_product_deletion' ), 10, 2 );
				add_action( 'woocommerce_before_delete_product_variation', array( $this, 'delete_batches_on_variation_deletion' ), 10, 1 );
			}

			if ( ! empty( WPO_WCPBN()->classes['settings']->options->get_option( 'display_batch_data_in_product_pages' ) ) && WPO_WCPBN()->classes['settings']->options->get_option( 'display_batch_data_in_product_pages' ) != 'no' ) {
				add_action( WPO_WCPBN()->classes['settings']->options->get_option( 'display_batch_data_in_product_pages_location' ), array( $this, 'display_batch_data_in_product_pages' ) );
				add_action( 'wp_ajax_wcpbn-on-variation-selection', array( $this, 'ajax_on_variation_selection' ) );
				add_action( 'wp_ajax_nopriv_wcpbn-on-variation-selection', array( $this, 'ajax_on_variation_selection' ) );
			}

			add_action( 'woocommerce_product_options_stock', array( $this, 'mirror_product_simple_batches_stock_setting' ), 999 );
			add_action( 'woocommerce_variation_options_pricing', array( $this, 'mirror_product_variation_batches_stock_setting' ), 999, 3 );
		}

		/**
		 * Batch numbers data for simple/bundle/subscription product.
		 *
		 * @return void
		 */
		public function batch_numbers_product_data() {
			$product_id = get_the_ID();

			// WPML compatibility
			$product_id = wpo_wcpbn_wpml_object_id( $product_id );
			$product    = wc_get_product( $product_id );

			if ( empty( $product ) ) {
				return;
			}

			if ( ! in_array( $product->get_type(), array( 'simple', 'bundle', 'subscription' ) ) ) {
				return;
			}

			// DB Query to get all batch numbers for simple product
			$args = array(
				'shared_ids' => [ intval( $product->get_id() ) ],
				'order_by'   => $this->orderby,
				'order'      => 'ASC',
			);
			$product_batch_numbers = wpo_wcpbn_get_batches( $args );

			// Get product status array
			$product_status = $this->product_status;

			// pass stock locations setting status
			$stock_locations_enabled = wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'stock_locations' ) );
			
			// pass batch units
			$batch_units_enabled = wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'batch_units' ) );

			// Include template
			include_once( WPO_WCPBN_TEMPLATES_PATH.'/template-wcpbn-product-data.php' );
		}

		/**
		 * Batch numbers data for variable product.
		 *
		 * @return void
		 */
		public function batch_numbers_product_data_variation( $loop, $variation_data, $variation ) {
			if ( empty( $variation ) ) {
				return;
			}

			// WPML compatibility
			$product_id   = wpo_wcpbn_wpml_object_id( $variation->post_parent );
			$variation_id = wpo_wcpbn_wpml_object_id( $variation->ID );

			// DB Query to get all batch numbers for variation product
			$query_args = array(
				'shared_ids' => [ intval( $variation_id ) ],
				'order_by'   => $this->orderby,
				'order'      => 'ASC',
			);
			$product_batch_numbers = wpo_wcpbn_get_batches( $query_args );

			// Get product status array
			$product_status = $this->product_status;

			// pass stock locations setting status
			$stock_locations_enabled = wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'stock_locations' ) );
			
			// pass batch units
			$batch_units_enabled = wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'batch_units' ) );

			// Include template
			include( WPO_WCPBN_TEMPLATES_PATH.'/template-wcpbn-product-data.php' );
		}

		/**
		 * CRUD operations for batch numbers in simple product.
		 *
		 * @return void
		 */
		public function crud_batch_numbers_product( $post_id ) {
			if ( isset( $_POST['post_id'] ) ) {
				$post_id = $_POST['post_id'];
			}

			if ( empty( $post_id ) ) {
				return;
			}

			$this->crud_batch_numbers( $post_id, null );
		}

		/**
		 * CRUD operations for batch numbers in variations.
		 *
		 * @return void
		 */
		public function crud_batch_numbers_product_variation( $variation_id, $i ) {
			$this->crud_batch_numbers( null, $variation_id );
		}

		private function crud_batch_numbers( $post_id, $variation_id ) {
			// Process POST
			if ( $_POST ) {

				// grab the correct ID
				$_id = ! empty( $post_id ) ? $post_id : $variation_id;

				$product = wc_get_product( $_id );
				if ( empty( $product ) ) {
					return;
				}

				if ( isset( $_POST['wpo-wcpbn-batch-number-id'][$_id] ) ) {

					$total_rows = count( $_POST['wpo-wcpbn-batch-number-id'][$_id] );

					for ( $i = 0; $i < $total_rows; $i++ ) {
						$j = $_POST['wpo-wcpbn-batch-number-id'][$_id][$i];
						
						// reset previous batch data
						$previous_batch_data = '';

						// Check if it's row from the database
						if ( $_POST['wpo-wcpbn-batch-number-id'][$_id][$i][0] != 0 ) {
							$data[$i]['id'] = $batch_id = sanitize_text_field( $_POST['wpo-wcpbn-batch-number-id'][$_id][$i] );
							$previous_batch_data = wpo_wcpbn_get_batches( [ 'id' => $batch_id ] );
							if ( ! empty( $previous_batch_data ) ) {
								$previous_batch_data = reset( $previous_batch_data );
							}
						}

						if ( isset( $_POST['wpo-wcpbn-table-expirydate'][$j] ) ) {
							$data[$i]['date_expiry'] = sanitize_text_field( $_POST['wpo-wcpbn-table-expirydate'][$j] );
						}
						if ( isset( $_POST['wpo-wcpbn-table-batchnumber'][$j] ) ) {
							$data[$i]['batch_number'] = sanitize_text_field( $_POST['wpo-wcpbn-table-batchnumber'][$j] );
						}
						if ( isset( $_POST['wpo-wcpbn-table-supplier'][$j] ) ) {
							$data[$i]['supplier'] = wp_kses( stripslashes( esc_attr( $_POST['wpo-wcpbn-table-supplier'][$j] ) ), array());
						}
						if ( isset( $_POST['wpo-wcpbn-table-location'][$j] ) ) {
							$data[$i]['location'] = wp_kses( stripslashes( esc_attr( $_POST['wpo-wcpbn-table-location'][$j] ) ), array());
						}
						if ( isset( $_POST['wpo-wcpbn-table-quantity'][$j] ) ) {
							$data[$i]['quantity'] = wpo_wcpbn_force_stock_amount_type( sanitize_text_field( $_POST['wpo-wcpbn-table-quantity'][$j] ) );
						}
						if ( isset( $_POST['wpo-wcpbn-table-quantity-available'][$j] ) ) {
							$data[$i]['quantity_available'] = wpo_wcpbn_force_stock_amount_type( sanitize_text_field( $_POST['wpo-wcpbn-table-quantity-available'][$j] ) );
						} else {
							$data[$i]['quantity_available'] = wpo_wcpbn_force_stock_amount_type( $data[$i]['quantity'] );
						}
						if ( isset( $_POST['wpo-wcpbn-table-units'][$j] ) ) {
							$data[$i]['units'] = absint( $_POST['wpo-wcpbn-table-units'][$j] );
						}
						if ( isset( $_POST['wpo-wcpbn-table-status'][$j] ) ) {
							$data[$i]['status'] = sanitize_text_field( $_POST['wpo-wcpbn-table-status'][$j] );
							if ( isset( $data[$i]['quantity_available'] ) && $data[$i]['quantity_available'] == 0 ) {
								$data[$i]['status'] = 'archived';
							}
						}

						// attach the current user ID
						$data[$i]['user_id'] = get_current_user_id();

						// reset email notifications if expire date changed
						if ( ! empty( $previous_batch_data ) && isset( $data[$i]['date_expiry'] ) && date_i18n( 'Y-m-d', strtotime( $previous_batch_data->date_expiry ) ) != $data[$i]['date_expiry'] ) {
							$notified_batches = get_option( 'wcpbn_date_expiry_global_notification_batches', array() );
							if ( ! empty( $notified_batches ) && isset( $data[$i]['id'] ) && in_array( $data[$i]['id'], $notified_batches ) ) {
								$update = false;
								foreach ( $notified_batches as $key => $value ) {
									if ( $value == $data[$i]['id'] ) {
										unset( $notified_batches[$key] );
										$update = true;
									}
								}
								if ( $update ) {
									update_option( 'wcpbn_date_expiry_global_notification_batches', $notified_batches );
								}
							}
						}

						// Check if it's to insert or update
						if ( ! empty( $previous_batch_data ) ) {
							// if we are adding more quantity to an existing batch, let's change the status to 'active'
							if ( $data[$i]['quantity_available'] > 0 && ! isset( $data[$i]['quantity'] ) && isset( $data[$i]['status'] ) && $data[$i]['status'] == 'archived' ) {
								$data[$i]['status'] = 'active';
							}
							
							$data_to_update = $data[$i];
							unset( $data_to_update['id'] );
							$query = wpo_wcpbn_update_batch( $batch_id, $data_to_update );
						} else {
							$data[$i]['shared_ids'] = [ intval( $_id ) ];    // attach the current product ID
							$data[$i]['date_created'] = date("Y-m-d H:i:s"); // only add date created if the batch is new
							unset( $data[$i]['id'] );                        // just to make sure we don't have an ID here by mistake
							$query = wpo_wcpbn_insert_batch( $data[$i] );
						}

						// sync WC stock
						if ( $query ) {
							// specific for new batch
							if ( ! isset( $data[$i]['id'] ) ) {
								// if it's a new batch at this time should be already created and should be the last one added
								$product = wc_get_product( $_id );
								if ( ! empty( $product ) ) {
									$last_batch     = wpo_wcpbn_get_product_last_added_batch( $_id );
									$data[$i]['id']	= ! empty( $last_batch ) ? $last_batch->id : false;
									$new            = true;
								}
							// for existing batch
							} else {
								// for existing batches, include the previous data for quantities comparison
								if ( ! empty( $previous_batch_data ) ) {
									$data[$i]['previous_data'] = (array) $previous_batch_data;
								}
							}

							// if we have the batch ID let's sync
							if ( ! empty( $data[$i]['id'] ) ) {
								$new = ! empty( $new ) ? $new : false;
								$this->sync_wc_stock( $data[$i], $new );
							}
						}
					}

				}

				// save the offset notification value if exists
				$product_offset = wpo_wcpbn_get_product_current_offset_in_days( $_id );
				$offset         = ! empty( $_POST['wpo-wcpbn-notification-offset'] ) ? intval( $_POST['wpo-wcpbn-notification-offset'] ) : '';
				if ( ! empty( $offset ) && $offset != $product_offset ) {
					$product->update_meta_data( '_wcpbn_notification_offset', $offset );
				}
				
				// save the mirror stock with batches setting
				$mirroring = ! empty( $_POST['wpo-wcpbn-mirror-batches-stock'] ) && $_POST['wpo-wcpbn-mirror-batches-stock'] == 'yes' ? sanitize_text_field( $_POST['wpo-wcpbn-mirror-batches-stock'] ) : 'no';
				$product->update_meta_data( '_wcpbn_mirror_batches_stock', $mirroring );

				// save meta to product
				$product->save_meta_data();
			}
		}

		public function sync_wc_stock( $data, $new = false, $deletion = false ) {
			$option = wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'sync_wc_stock' ) );

			if ( $option && is_array( $data ) ) {
				if ( isset( $data['id'] ) && $data['id'] != 0 ) {
					$batch = wpo_wcpbn_get_batches( [ 'id' => $data['id'] ] );
					if ( empty( $batch ) ) {
						return;
					}
					$batch = is_array( $batch ) ? reset( $batch ) : $batch;
					if ( ! empty( $batch ) ) {
						$all_shared_ids = wpo_wcpbn_get_batch_shared_product_ids( $batch->id );
						
						if ( empty( $all_shared_ids ) ) {
							return;
						}
						
						foreach ( $all_shared_ids as $product_id ) {
							$product = wc_get_product( $product_id );
							if ( $product ) {
								wpo_wcpbn_sync_product_stock_with_batches( $product );
							}
						}
					}
				}

			}
		}

		public function delete_batch_number_row() {
			// Check if it's delete
			if ( isset( $_POST['batch_id'] ) ) {
				$data     = stripslashes_deep( $_POST );
				$batch_id = sanitize_text_field( $data['batch_id'] );

				// delete from DB
				wpo_wcpbn_delete_batch( $batch_id, $data );
			}
		}

		public function new_batch_number_row_html() {	
			$random_unique_id        = '0'.$this->generate_random_string( 8 );
			$stock_locations_enabled = wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'stock_locations' ) );
			$batch_units_enabled     = wc_string_to_bool( WPO_WCPBN()->classes['settings']->options->get_option( 'batch_units' ) );
			?>
				<tr>
					<td class="column-expirydate">
						<input class="wpo-wcpbn-batch-number-id" type="hidden" name="" value="<?= $random_unique_id; ?>">
						<input type="date" name="wpo-wcpbn-table-expirydate[<?= $random_unique_id; ?>]" >
					</td>
					<td class="column-batchnumber"><input class="wpo-wcpbn-table-batchnumber" type="text" name="wpo-wcpbn-table-batchnumber[<?= $random_unique_id; ?>]" placeholder="<?= __( 'Batch number', 'woocommerce-product-batch-numbers' ); ?>" required></td>
					<td class="column-supplier"><input class="wpo-wcpbn-table-supplier" type="text" name="wpo-wcpbn-table-supplier[<?= $random_unique_id; ?>]" pattern="[A-Za-z0-9 ]+" placeholder="<?= __( 'Supplier', 'woocommerce-product-batch-numbers' ); ?>"></td>
					<?php if ( $stock_locations_enabled ) : ?>
						<td class="column-location"><input class="wpo-wcpbn-table-location" type="text" name="wpo-wcpbn-table-location[<?= $random_unique_id; ?>]" pattern="[A-Za-z0-9 ]+" placeholder="<?= __( 'Location', 'woocommerce-product-batch-numbers' ); ?>"></td>
					<?php endif; ?>
					<td class="column-quantities"><input type="number" name="wpo-wcpbn-table-quantity[<?= $random_unique_id; ?>]" placeholder="<?= __( 'Quantity', 'woocommerce-product-batch-numbers' ); ?>" required></td>
					<?php if ( $batch_units_enabled ) : ?>
						<td class="column-units"><input class="wpo-wcpbn-table-units" type="number" name="wpo-wcpbn-table-units[<?= $random_unique_id; ?>]" min="1" value="1" required></td>
					<?php endif; ?>
					<td class="column-status">
						<select disabled>
							<option selected><?php _e( 'Draft', 'woocommerce-product-batch-numbers' ); ?></option>
						</select>
					</td>
					<td class="column-actions">
						<a class="wpo-wcpbn-table-delete-new-row"><span class="dashicons dashicons-trash"></span></a>
					</td>
				</tr>
			<?php
		}

		public function get_last_added_batch() {
			if ( isset( $_POST ) && $_POST['action'] == 'get_last_added_batch' && ! empty( $_POST['id'] ) ) {
				$id      = absint( $_POST['id'] );
				$product = wc_get_product( $id );

				if ( empty( $product ) ) {
					wp_send_json_error( array(
						'message' => __( 'Product not found!', 'woocommerce-product-batch-numbers' ),
					) );
				}

				$last_batch = wpo_wcpbn_get_product_last_added_batch( $product->get_id(), true );

				if ( ! empty( $last_batch ) ) {
					// format quantities for JS
					$last_batch->quantity           = wpo_wcpbn_format_stock_amount( $last_batch->quantity );
					$last_batch->quantity_available = wpo_wcpbn_format_stock_amount( $last_batch->quantity_available );

					$last_batch = $last_batch;
					wp_send_json_success( compact( 'last_batch' ) );
				} else {
					wp_send_json_error( array(
						'message' => __( 'Last batch not found!', 'woocommerce-product-batch-numbers' ),
					) );
				}
			}
		}

		public function supplier_names() {
			if ( isset( $_POST['search'] ) ) {
				$result = wpo_wcpbn_get_batch_supplier_by_name( esc_attr( $_POST['search'] ) );

				$supplier_names = array();
				foreach ( $result as $key => $name ) {
					$supplier_names[] = array(
						'value' => $name->supplier,
						'label' => $name->supplier,
					);
				}

				echo json_encode( $supplier_names );
			}
			exit;
		}

		public function batch_number_check() {
			if ( isset( $_POST['search'] ) ) {
				$search        = sanitize_text_field( $_POST['search'] );
				$results       = wpo_wcpbn_get_batches( array( 'batch_number' => $search ) );
				$total_results = count( $results );

				if ( $total_results > 0 ) {
					wp_send_json_success( array(
						'duplicated' => true,
						'tooltip'    => __( 'The batch number is not unique!', 'woocommerce-product-batch-numbers' ),
					) );
				} else {
					wp_send_json_success( array(
						'duplicated' => false,
						'tooltip'    => '',
					) );
				}
			} else {
				wp_send_json_error();
			}
			
			wp_die();
		}

		private function generate_random_string( $length = 5 ) {
			$characters       = '0123456789';
			$charactersLength = strlen( $characters );
			$randomString     = '';

			for ( $i = 0; $i < $length; $i++ ) {
				$randomString .= $characters[rand( 0, $charactersLength - 1 )];
			}
			return $randomString;
		}

		public function sync_product_simple_stock_button() {
			global $post;

			$product_id = wpo_wcpbn_wpml_object_id( $post->ID ); // WPML compatibility
			$class		= 'wpo-wcpbn-product-simple-stock-sync';
			$product	= wc_get_product( $product_id );

			if ( empty( $product ) || ! $product->managing_stock() ) {
				return;
			}

			$type = $product->get_type();

			// Include template
			include( WPO_WCPBN_TEMPLATES_PATH.'/template-wcpbn-product-stock-sync.php' );
		}

		public function sync_product_variation_stock_button( $loop, $variation_data, $variation ) {
			global $post;

			if ( empty( $variation ) ) {
				return;
			}

			$product_id	= wpo_wcpbn_wpml_object_id( $variation->ID ); // WPML compatibility
			$class		= 'wpo-wcpbn-product-variation-stock-sync';
			$product	= wc_get_product( $product_id );
			
			if ( empty( $product ) || ! $product->managing_stock() ) {
				return;
			}
			
			$type = $product->get_type();

			// Include template
			include( WPO_WCPBN_TEMPLATES_PATH.'/template-wcpbn-product-stock-sync.php' );
		}

		public function sync_product_stock_ajax() {
			if ( isset( $_POST ) && $_POST['action'] == 'sync_product_wc_stock' && ! empty( $_POST['product_id'] ) ) {
				$product_id = wpo_wcpbn_wpml_object_id( sanitize_text_field( $_POST['product_id'] ) ); // WPML compatibility
				$product	= wc_get_product( $product_id );
				
				if ( empty( $product ) || ! is_object( $product ) ) {
					wp_send_json_error( array( 'message' => __( 'Product object not found!', 'woocommerce-product-batch-numbers' ) ) );
				}

				$sync = wpo_wcpbn_sync_product_stock_with_batches( $product );

				if ( $sync ) {
					wp_send_json_success( array( 'message' => __( 'WooCommerce product stock synced with batches!', 'woocommerce-product-batch-numbers' ) ) );
				} else {
					wp_send_json_error( array( 'message' => __( 'No stock in batches to be synced!', 'woocommerce-product-batch-numbers' ) ) );
				}
			}
		}

		public function mirror_product_simple_batches_stock_setting() {
			global $post;

			$product_id = wpo_wcpbn_wpml_object_id( $post->ID ); // WPML compatibility
			$product	= wc_get_product( $product_id );

			if ( empty( $product ) || ! $product->managing_stock() ) {
				return;
			}
			
			$mirroring = wpo_wcpbn_is_product_mirroring_batches_stock( $product_id );

			woocommerce_wp_checkbox(
				array(
					'id'            => 'wpo-wcpbn-mirror-batches-stock',
					'value'         => $mirroring ? 'yes' : 'no',
					'wrapper_class' => 'show_if_simple show_if_variable wcpbn_mirror_stock_with_batches',
					'label'         => __( 'Mirror the stock of the batches', 'woocommerce-product-batch-numbers' ),
					'description'   => __( "This will mirror the same stock amount for the product and all it's batches, based on the batch with the highest value.", 'woocommerce-product-batch-numbers' ) . wc_help_tip( __( 'Also, it will mirror the same stock amount for all of the batches when consuming or restoring.', 'woocommerce-product-batch-numbers' ) ),
				)
			);
		}

		public function mirror_product_variation_batches_stock_setting( $loop, $variation_data, $variation ) {
			global $post;

			if ( empty( $variation ) ) {
				return;
			}

			$product_id	= wpo_wcpbn_wpml_object_id( $variation->ID ); // WPML compatibility
			$product	= wc_get_product( $product_id );
			
			if ( empty( $product ) || ! $product->managing_stock() ) {
				return;
			}
			
			$mirroring = wpo_wcpbn_is_product_mirroring_batches_stock( $product_id );

			echo '<div>';
			woocommerce_wp_checkbox(
				array(
					'id'            => 'wpo-wcpbn-mirror-batches-stock',
					'value'         => $mirroring ? 'yes' : 'no',
					'wrapper_class' => 'form-row form-row-last wcpbn_mirror_stock_with_batches',
					'label'         => __( 'Mirror the stock of the batches', 'woocommerce-product-batch-numbers' ),
					'description'   => __( "This will mirror the same stock amount for the product and all it's batches, based on the batch with the highest value.", 'woocommerce-product-batch-numbers' ) . wc_help_tip( __( 'Also, it will mirror the same stock amount for all of the batches when consuming or restoring.', 'woocommerce-product-batch-numbers' ) ),
				)
			);
			echo '</div>';
		}

		public function delete_batches_on_product_deletion( $post_id, $post ) {
			if ( ! empty( $post_id ) && get_post_type( $post_id ) == 'product' ) {
				$product_batches = wpo_wcpbn_get_batches( [ 'shared_ids' => [ intval( $post_id ) ] ] );
				if ( ! empty( $product_batches ) ) {
					foreach ( $product_batches as $batch ) {
						$all_shared_ids = wpo_wcpbn_get_batch_shared_product_ids( $batch->id );
						if ( count( $all_shared_ids ) == 1 ) {
							wpo_wcpbn_delete_batch( $batch->id, [ 'product_id' => $post_id ] );
						}
					}
				}
			}
		}

		public function delete_batches_on_variation_deletion( $variation_id ) {
			if ( ! empty( $variation_id ) ) {
				$product_batches = wpo_wcpbn_get_batches( [ 'shared_ids' => [ intval( $variation_id ) ] ] );
				if ( ! empty( $product_batches ) ) {
					foreach ( $product_batches as $batch ) {
						$all_shared_ids = wpo_wcpbn_get_batch_shared_product_ids( $batch->id );
						if ( count( $all_shared_ids ) == 1 ) {
							wpo_wcpbn_delete_batch( $batch->id, [ 'product_id' => $variation_id ] );	
						}
					}
				}
			}
		}

		public function display_batch_data_in_product_pages() {
			global $product;

			if ( ! is_product() ) {
				return;
			}

			if ( ! is_object( $product) ) {
				$product = wc_get_product( wpo_wcpbn_wpml_object_id( get_the_ID() ) );
			}

			$this->product_batch_data( $product, true );
		}

		public function ajax_on_variation_selection() {
			if ( empty( $_POST['nonce'] ) ) {
				wp_send_json_error( array( 'message' => __( 'Nonce is empty!', 'woocommerce-product-batch-numbers' ) ) );
			}

			wp_verify_nonce( $_POST['nonce'], 'wpo-wcpbn' );

			if ( isset( $_POST['action'] ) && $_POST['action'] == 'wcpbn-on-variation-selection' && ! empty( $_POST['variation_id'] ) ) {
				$variation_product = wc_get_product( wpo_wcpbn_wpml_object_id( sanitize_text_field( $_POST['variation_id'] ) ) );
				$hook              = WPO_WCPBN()->classes['settings']->options->get_option( 'display_batch_data_in_product_pages_location' );
				$css_target        = $this->variable_product_html_elem_hook_matching( $hook );
				$html              = $this->product_batch_data( $variation_product );

				wp_send_json_success( compact( 'css_target', 'html' ) );
			} else {
				wp_send_json_error( array( 'message' => __( 'An error occurred while processing this request.', 'woocommerce-product-batch-numbers' ) ) );
			}

			wp_die();
		}

		public function variable_product_html_elem_hook_matching( $hook ) {
			if ( empty( $hook ) ) {
				return array();
			}

			$hook_match = apply_filters( 'wpo_wcpbn_variable_product_html_elem_hook_matching' , array(
				'woocommerce_single_product_summary' => array(
					'css'    => 'div.summary h1.product_title',
					'inject' => 'after',
				),
				'woocommerce_before_add_to_cart_form' => array(
					'css'    => 'form.cart',
					'inject' => 'before',
				),
				'woocommerce_after_add_to_cart_form' => array(
					'css'    => 'form.cart',
					'inject' => 'after',
				),
				'woocommerce_before_add_to_cart_button' => array(
					'css'    => 'button.single_add_to_cart_button',
					'inject' => 'before',
				),
				'woocommerce_after_add_to_cart_button' => array(
					'css'    => 'button.single_add_to_cart_button',
					'inject' => 'after',
				),
				'woocommerce_before_add_to_cart_quantity' => array(
					'css'    => 'input.qty',
					'inject' => 'before',
				),
				'woocommerce_after_add_to_cart_quantity' => array(
					'css'    => 'input.qty',
					'inject' => 'after',
				),
				'woocommerce_product_meta_start' => array(
					'css'    => 'div.product_meta',
					'inject' => 'before',
				),
				'woocommerce_product_meta_end' => array(
					'css'    => 'div.product_meta',
					'inject' => 'after',
				),
			), $hook );

			if ( isset( $hook_match[$hook] ) ) {
				return $hook_match[$hook];
			} else {
				return array();
			}
		}

		public function product_batch_data( $product, $print = false ) {
			$data_format      = WPO_WCPBN()->classes['settings']->options->get_option( 'display_batch_data_in_product_pages' );
			$include_quantity = apply_filters( 'wpo_wcpbn_display_quantity_in_product_pages', false );
			$time_window      = WPO_WCPBN()->classes['settings']->options->get_option( 'display_batch_data_in_product_pages_time_window' );
			$days_from        = ( ! empty( $time_window ) && wc_string_to_bool( $time_window['enable'] ) && ! empty( $time_window['from'] ) ) ? $time_window['from'] : '';
			$days_to          = ( ! empty( $time_window ) && wc_string_to_bool( $time_window['enable'] ) && ! empty( $time_window['to'] ) ) ? $time_window['to'] : '';
			$limit            = WPO_WCPBN()->classes['settings']->options->get_option( 'display_batch_data_in_product_pages_limit' );

			if ( ! empty( $product ) && ! in_array( $product->get_type(), array( 'variable', 'variable-subscription' ) ) ) {
				if ( $print ) {
					wpo_wcpbn_print_product_batch_data_formatted( $product->get_id(), $data_format, $include_quantity, true, $days_from, $days_to, $limit );
				} else {
					return wpo_wcpbn_get_product_batch_data_formatted( $product->get_id(), $data_format, $include_quantity, true, $days_from, $days_to, $limit );
				}
			}
			
			if ( $print ) {
				echo '';
			} else {
				return '';
			}
		}

	}

}