<?php
/**
 * REST API
 *
 * @since 1.1.4
 */

namespace WPO\WC\Product_Batch_Numbers;

/**
 * If this file is called directly, abort.
 *
 */
if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if ( ! class_exists( 'wcpbnRest' ) ) {

	class wcpbnRest {

		protected static $_instance = null;

		/**
		 * Instance
		 */
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}
		
		/**
		 * Constructor
		 */
		public function __construct() {
			add_action( 'rest_api_init', array( $this, 'rest_api_init' ) );
		}

		/**
		 * Create REST field
		 *
		 * @return void
		 */
		public function rest_api_init() {
			register_rest_field( 'product', 'batches', array(
				'get_callback'    => array( $this, 'product_get_callback' ),
				'update_callback' => array( $this, 'product_update_callback' ),
				'schema'          => null,
			) );

			register_rest_field( 'product_variation', 'batches', array(
				'get_callback'    => array( $this, 'product_get_callback' ),
				'update_callback' => array( $this, 'product_update_callback' ),
				'schema'          => null,
			) );
		}

		/**
		 * Get product callback
		 *
		 * @param $post
		 * @return array
		 */
		public function product_get_callback( $post, $attr, $request, $object_type ) {
			if ( $request && 'GET' !== $request->get_method() ) {
				return array();
			}

			$pid = wpo_wcpbn_wpml_object_id( absint( $post['id'] ) );

			return wpo_wcpbn_get_batches( array( 'shared_ids' => absint( $pid ) ) );
		}

		/**
		 * Update product callback
		 *
		 * @param $post
		 * @param $request
		 */
		public function product_update_callback( $values, $post, $attr, $request, $object_type ) {
			if ( $request && 'PUT' !== $request->get_method() ) {
				return;
			}

			$pid = wpo_wcpbn_wpml_object_id( $post->get_id() );
			
			$default_keys = array(
				'date_created',
				'date_expiry',
				'user_id',
				'batch_number',
				'supplier',
				'location',
				'quantity',
				'quantity_available',
				'status',
				'shared_ids',
			);

			if ( ! empty( $values ) && is_array( $values ) ) {				
				foreach ( $values as $batch ) {
					if ( 2 > count( $batch ) ) {
						continue;
					}
					
					$id = isset( $batch['id'] ) ? absint( $batch['id'] ) : null;
					
					// maybe delete batch
					if ( ! empty( $id ) && ( isset( $batch['delete'] ) || isset( $batch['force_delete'] ) ) ) {
						$force = isset( $batch['force_delete'] ) ? true : false;
						wpo_wcpbn_delete_batch( $id, array(), $force );
						continue;
					}
					
					// process data
					$data = array();
					foreach ( $default_keys as $key ) {
						switch ( $key ) {
							case 'shared_ids':
								if ( ! empty( $batch[ $key ] ) ) {
									if ( is_array( $batch[ $key ] ) ) {
										$data[ $key ] = $batch[ $key ];
									} elseif ( is_numeric( $batch[ $key ] ) ) {
										$data[ $key ] = array( absint( $batch[ $key ] ) );
									} else {
										$data[ $key ] = array( absint( $pid ) );
									}
								} else {
									$data[ $key ] = array( absint( $pid ) );
								}
								break;
							case 'date_created':
							case 'date_expiry':
							case 'batch_number':
							case 'supplier':
							case 'location':
							case 'status':
								if ( isset( $batch[ $key ] ) ) {
									$data[ $key ] = sanitize_text_field( $batch[ $key ] );
								}
								break;
							case 'quantity':
							case 'quantity_available':
								if ( isset( $batch[ $key ] ) ) {
									$data[ $key ] = wpo_wcpbn_force_stock_amount_type( $batch[ $key ] );
								}
								break;
							case 'user_id':
								if ( isset( $batch[ $key ] ) ) {
									$data[ $key ] = absint( $batch[ $key ] );
								}
								break;
						}
					}
					
					// insert batch
					if ( empty( $id ) ) {
						$data['date_created'] = date( 'Y-m-d H:i:s' );
						if ( empty( $data['user_id'] ) ) {
							$data['user_id'] = apply_filters( 'wpo_wcpbn_rest_api_user_id', 1 );
						}
						if ( apply_filters( 'wpo_wcpbn_rest_auto_set_quantity_available', '__return_true' ) && isset( $data['quantity'] ) && ( ! isset( $data['quantity_available'] ) || $data['quantity_available'] > $data['quantity'] ) ) {
							$data['quantity_available'] = $data['quantity'];
						}
						if ( ! isset( $data['quantity'] ) || 0 === $data['quantity'] ) {
							$data['status'] = 'archived';
						}
						if ( empty( $data['batch_number'] ) ) {
							$data['batch_number'] = substr( md5( rand() ), 0, 10 );
						}
						wpo_wcpbn_insert_batch( $data );
						
					// update batch
					} elseif ( ! empty( $id ) && ! empty( $data ) ) {
						if ( ( isset( $data['shared_ids'] ) && 1 < count( $data ) ) || ( ! isset( $data['shared_ids'] ) && 0 < count( $data ) ) ) {
							wpo_wcpbn_update_batch( $id, $data );
						} else {
							continue;
						}
						
					// nothing to do
					} else {
						continue;
					}
				}
			}
			
			// sync WC product stock
			wpo_wcpbn_sync_product_stock_with_batches( $pid );
		}

	}

}
