<?php

namespace WPO\WC\Product_Batch_Numbers;

use WPO\WC\Product_Batch_Numbers\Settings\wcpbnSettingsOverview;
use WPO\WC\Product_Batch_Numbers\Settings\wcpbnSettingsOptions;
use WPO\WC\Product_Batch_Numbers\Settings\wcpbnSettingsExport;
use WPO\WC\Product_Batch_Numbers\Settings\wcpbnSettingsRest;
use WPO\WC\Product_Batch_Numbers\Settings\wcpbnSettingsTools;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if ( ! class_exists( 'wcpbnSettings' ) ) {

	class wcpbnSettings {

		public           $overview    = null;
		public           $options     = null;
		public           $export      = null;
		public           $tools       = null;
		public           $rest        = null;
		public           $hook_suffix = null;
		protected static $_instance   = null;

		/**
		 * Instance
		 */
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		/**
		 * Construct
		 */
		public function __construct() {
			$this->setup_classes();
			
			add_action( 'admin_menu', array( $this, 'settings_submenu_page' ), 99 );
			add_filter( 'plugin_action_links_'.WPO_WCPBN()::$plugin_basename, array( $this, 'add_settings_link' ) );
		}

		/**
		 * Instantiate additional settings classes
		 */
		private function setup_classes() {
			$this->overview = wcpbnSettingsOverview::instance();
			$this->options  = wcpbnSettingsOptions::instance();
			$this->export   = wcpbnSettingsExport::instance();
			$this->tools    = wcpbnSettingsTools::instance();
			$this->rest     = wcpbnSettingsRest::instance();
		}

		/**
		 * Creates a settings submenu page.
		 *
		 * @return void
		 */
		public function settings_submenu_page() {
			$this->hook_suffix = add_submenu_page(
				'edit.php?post_type=product',
				__( 'Batch Numbers', 'woocommerce-product-batch-numbers' ),
				__( 'Batch Numbers', 'woocommerce-product-batch-numbers' ),
				'manage_woocommerce',
				'wpo-batch-numbers',
				array( $this, 'settings_callback' )
			);

			// load overview list screen options
			if ( ! is_null( $this->overview ) && is_callable( array( $this->overview, 'screen_options' ) ) ) {
				add_action( "load-{$this->hook_suffix}", array( $this->overview, 'screen_options' ) );
			}
		}

		/**
		 * Initiates the plugin settings page tabs
		 * 
		 * @return array
		 */
		public function tabs() {
			$default_tabs = array(
				'overview' => __( 'Overview', 'woocommerce-product-batch-numbers' ),
				'options'  => __( 'Options', 'woocommerce-product-batch-numbers' ),
				'export'   => __( 'Export', 'woocommerce-product-batch-numbers' ),
				'tools'    => __( 'Tools', 'woocommerce-product-batch-numbers' ),
				'rest'     => __( 'REST', 'woocommerce-product-batch-numbers' ),
			);
			
			return apply_filters( 'wpo_wcpbn_settings_tabs', $default_tabs );
		}

		/**
		 * Settings submenu page callback function.
		 *
		 * @return void
		 */
		public function settings_callback() {
			settings_errors();
			$current_tab     = $_REQUEST['tab'] = $_REQUEST['tab'] ?? 'overview';
			$sections        = apply_filters( "wpo_wcpbn_{$current_tab}_settings_sections", array() );
			$current_section = $_REQUEST['section'] = $_REQUEST['section'] ?? key( $sections );
			?>
			<div class="wrap">
				<div class="wpo_wcpbn_settings">
					<h2><?php _e( 'Product Batch Numbers', 'woocommerce-product-batch-numbers' ); ?></h2>
					<!-- Main menu -->
					<?php do_action( 'wpo_wcpbn_before_settings_tabs', $current_tab, $current_section ); ?>
					<h2 class="nav-tab-wrapper">
						<?php
						foreach ( $this->tabs() as $key => $label ) {
							$class = 'nav-tab';
							if ( $current_tab == $key ) {
								$class .= ' nav-tab-active';
							}
							$tab_url = add_query_arg( array(
								'page' => 'wpo-batch-numbers',
								'tab'  => $key,
							), admin_url( 'edit.php?post_type=product' ) );
							printf( '<a href="%s" class="%s">%s</a>', esc_url( $tab_url ), $class, $label );
						}
						?>
					</h2>
					<?php do_action( 'wpo_wcpbn_after_settings_tabs', $current_tab, $current_section ); ?>
					<div class="wpo_wcpbn_settings_container">
						<?php do_action( 'wpo_wcpbn_before_settings_tab_content', $current_tab, $current_section ); ?>
						<?php do_action_deprecated( 'wpo_wcpbn_before_settings_page', array( $current_tab, $current_section ), '1.3.1', 'wpo_wcpbn_before_settings_tab_content' ); ?>
						<?php do_action( 'wpo_wcpbn_settings_tab_content', $current_tab, $current_section ); ?>
						<?php do_action( 'wpo_wcpbn_after_settings_tab_content', $current_tab, $current_section ); ?>
						<?php do_action_deprecated( 'wpo_wcpbn_after_settings_page', array( $current_tab, $current_section ), '1.3.1', 'wpo_wcpbn_after_settings_tab_content' ); ?>
					</div>
				</div>
			</div>
			<?php
		}

		/**
		 * Adds setting link to plugin in plugins list page.
		 *
		 * @return void
		 */
		public function add_settings_link( $links ) {
			$links[] = '<a href="' . admin_url( 'edit.php?post_type=product&page=wpo-batch-numbers' ) . '">'. __( 'Settings', 'woocommerce' ) . '</a>';
			return $links;
		}

	}

}