<?php
/**
 * Shortcodes
 *
 * @since 1.1.0
 */

namespace WPO\WC\Product_Batch_Numbers;

/**
 * If this file is called directly, abort.
 *
 */
if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if ( ! class_exists( 'wcpbnShortcodes' ) ) {

	class wcpbnShortcodes {

		protected        $orderby;
		protected static $_instance = null;

		/**
		 * Instance
		 */
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		/**
		 * Constructor
		 */
		public function __construct() {
			add_shortcode( 'wcpbn_product_next_batch_expiry_date', array( $this, 'display_batch_expiry_date' ) );
			add_shortcode( 'wcpbn_product_batch_numbers', array( $this, 'display_batch_number' ) );

			// get batch order by priority
			$this->orderby = ( $batch_stock_reduction_priority = WPO_WCPBN()->classes['settings']->options->get_option( 'batch_prioritization' ) ) ? $batch_stock_reduction_priority : 'id';
		}

		public function display_batch_expiry_date( $atts ) {
			global $product;

			if ( ! is_product() ) {
				return;
			}

			if ( ! is_object( $product) ) {
				$product = wc_get_product( get_the_ID() );
			}

			// Default values
			$values = shortcode_atts( array(
				'show_days_before_expiry' => '',
				'hide_days_before_expiry' => '',
				'limit'                   => '',
			), $atts );

			if ( ! $values ) {
				return;
			}

			$values['text_before_date'] = __( 'Next batch expiry date for', 'woocommerce-product-batch-numbers' );

			if ( ! empty( $product ) ) {
				if ( in_array( $product->get_type(), array( 'variable', 'variable-subscription' ) ) ) {
					$children = $product->get_children();
					foreach ( $children as $children_id ) {
						wpo_wcpbn_print_product_batch_data_formatted( $children_id, 'date', false, true, $values['show_days_before_expiry'], $values['hide_days_before_expiry'], $values['limit'] );
					}
				} else {
					wpo_wcpbn_print_product_batch_data_formatted( $product->get_id(), 'date', false, true, $values['show_days_before_expiry'], $values['hide_days_before_expiry'], $values['limit'] );
				}
			}
		}

		public function display_batch_number( $atts ) {
			global $product;

			if ( ! is_product() ) {
				return;
			}

			if ( ! is_object( $product) ) {
				$product = wc_get_product( get_the_ID() );
			}

			// Default values
			$values = shortcode_atts( array(
				'show_expiry_date' => '',
				'show_quantity'    => '',
				'limit'            => '',
			), $atts );

			if ( ! $values ) {
				return;
			}

			if ( wc_string_to_bool( $values['show_expiry_date'] ) ) {
				$data_format = 'number+date';
			} else {
				$data_format = 'number';
			}

			if ( ! empty( $product ) ) {
				if ( in_array( $product->get_type(), array( 'variable', 'variable-subscription' ) ) ) {
					$children = $product->get_children();
					foreach ( $children as $children_id ) {
						wpo_wcpbn_print_product_batch_data_formatted( $children_id, $data_format, wc_string_to_bool( $values['show_quantity'] ), true, '', '', $values['limit'] );
					}
				} else {
					wpo_wcpbn_print_product_batch_data_formatted( $product->get_id(), $data_format, wc_string_to_bool( $values['show_quantity'] ), true, '', '', $values['limit'] );
				}
			}
		}

	}

}