<?php
/**
 * Plugin Name:          WooCommerce Product Batch Numbers
 * Plugin URI:           https://wpovernight.com/downloads/woocommerce-product-batch-numbers/
 * Description:          Add batch numbers to your WooCommerce Products and Variations.
 * Version:              3.1.4
 * Author:               WP Overnight
 * Author URI:           https://wpovernight.com
 * License:              GPLv2 or later
 * License URI:          https://opensource.org/licenses/gpl-license.php
 * Text Domain:          woocommerce-product-batch-numbers
 * Domain Path:          /languages
 * WC requires at least: 3.0
 * WC tested up to:      8.5
 */

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

if ( ! class_exists( 'WPO_WCPBN' ) ) {

	class WPO_WCPBN {

		public           $version          = '3.1.4';
		public           $version_setting  = 'wcpbn_plugin_version';
		public           $classes          = [];
		public           $plugin_dir_url;
		public           $plugin_dir_path;
		public           $assets_suffix;
		public static    $plugin_basename;
//		private          $updater;
		protected static $instance         = null;

		/**
		 * Class Constructor.
		 */
		public function __construct() {
			// Save plugin dir url to property
			$this->plugin_dir_url  = plugin_dir_url( __FILE__ );
			$this->plugin_dir_path = plugin_dir_path( __FILE__ );
			self::$plugin_basename = plugin_basename( __FILE__ );

			$this->assets_suffix = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';

			// Define plugin version constant
			$this->define( 'WPBATCHNUMBERS_VERSION', $this->version );

			// Load actions
			add_action( 'init', array( $this, 'load_textdomain' ) );
			add_action( 'admin_enqueue_scripts', array( $this, 'load_admin_assets' ) );
			add_action( 'wp_enqueue_scripts', array( $this, 'load_frontend_assets' ) );

			// Declare HPOS compatibility.
			add_action( 'before_woocommerce_init', array( $this, 'custom_order_tables_compatibility' ) );
		}

		/**
		 * Ensures only one instance of our plugin is loaded or can be loaded.
		 *
		 * @return object
		 */
		public static function instance() {
			if ( ! isset( self::$instance ) && ! ( self::$instance instanceof WPO_WCPBN ) ) {
				self::$instance = new self();

				// Load includes
				self::$instance->includes();
				// Load classes
				self::$instance->setup_classes();
			}

			return self::$instance;
		}

		/**
		 * Required additional files
		 *
		 * @return void
		 */
		private function includes() {
			include_once( plugin_dir_path( __FILE__ ) . 'src/wcpbn-functions.php' );
			include_once( plugin_dir_path( __FILE__ ) . 'src/wcpbn-deprecated-functions.php' );
			include_once( plugin_dir_path( __FILE__ ) . 'src/class-wcpbn-semaphore.php' );
		}

		/**
		 * Setup additional classes
		 *
		 * @return void
		 */
		private function setup_classes() {
			if ( $this->is_woocommerce_activated() === false ) {
				return;
				add_action( 'admin_notices', array ( $this, 'need_woocommerce_notice' ) );
				return;
			}

			$item_name    = 'WooCommerce Product Batch Numbers';
			$file         = __FILE__;
			$license_slug = 'woocommerce_product_batch_numbers_license';
			$version      = WPBATCHNUMBERS_VERSION;
			$author       = 'Alexandre Faustino';

			// load updater
			if ( class_exists( 'WPO_Updater' ) ) { // WP Overnight Sidekick plugin
				$this->updater = new WPO_Updater( $item_name, $file, $license_slug, $version, $author );
			} else { // bundled updater
				if ( ! class_exists( 'WPO_Update_Helper' ) ) {
					include_once( 'updater/update-helper.php' );
				}
				$this->updater = new WPO_Update_Helper( $item_name, $file, $license_slug, $version, $author );
			}

			// if no Sidekick and no license, show notice in plugin settings page
			if ( is_callable( array( $this->updater, 'license_is_active' ) ) && $this->updater->license_is_active() ) {
				add_action( 'wpo_wcpbn_after_settings_tabs', array( $this, 'no_active_license_message' ), 1, 2 );
			}

			$this->classes['logger']          = WPO\WC\Product_Batch_Numbers\wcpbnLogger::instance();
			$this->classes['admin_notices']   = WPO\WC\Product_Batch_Numbers\wcpbnAdminNotices::instance();
			$this->classes['database']        = WPO\WC\Product_Batch_Numbers\wcpbnDatabase::instance();
			$this->classes['settings']        = WPO\WC\Product_Batch_Numbers\wcpbnSettings::instance();
			$this->classes['batches']         = WPO\WC\Product_Batch_Numbers\wcpbnBatches::instance();
			$this->classes['product']         = WPO\WC\Product_Batch_Numbers\wcpbnProduct::instance();
			$this->classes['cart']            = WPO\WC\Product_Batch_Numbers\wcpbnCart::instance();
			$this->classes['order_items']     = WPO\WC\Product_Batch_Numbers\wcpbnOrderItems::instance();
			$this->classes['search']          = WPO\WC\Product_Batch_Numbers\wcpbnSearch::instance();
			$this->classes['shortcodes']      = WPO\WC\Product_Batch_Numbers\wcpbnShortcodes::instance();
			$this->classes['product_listing'] = WPO\WC\Product_Batch_Numbers\wcpbnProductListing::instance();
			$this->classes['order_listing']   = WPO\WC\Product_Batch_Numbers\wcpbnOrderListing::instance();
			$this->classes['rest']            = WPO\WC\Product_Batch_Numbers\wcpbnRest::instance();
			$this->classes['purchase_orders'] = WPO\WC\Product_Batch_Numbers\wcpbnPurchaseOrders::instance();

			// Database classes for backward compatibility.
			$this->classes['db_batches']         = WPO\WC\Product_Batch_Numbers\Database\wcpbnDbBatchNumbers::instance();
			$this->classes['db_shared_products'] = WPO\WC\Product_Batch_Numbers\Database\wcpbnDbSharedProducts::instance();
			$this->classes['db_purchase_orders'] = WPO\WC\Product_Batch_Numbers\Database\wcpbnDbPurchaseOrders::instance();

			WPO\WC\Product_Batch_Numbers\wcpbnInstall::instance();
			WPO\WC\Product_Batch_Numbers\wcpbnCsvExport::instance();
		}

		public function no_active_license_message( $current_tab, $current_section ) {
			if ( class_exists( 'WPO_Updater' ) ) {
				$activation_url = esc_url_raw( network_admin_url( 'admin.php?page=wpo-license-page' ) );
			} else {
				$activation_url = esc_url_raw( network_admin_url( 'plugins.php?s=WooCommerce+Product+Batch+Numbers' ) );
			}
			?>
			<div class="notice notice-warning inline">
				<p>
					<?php /* translators: click here */ ?>
					<?php printf(__( 'Your license has not been activated on this site, %s to enter your license key.', 'woocommerce-product-batch-numbers' ), '<a href="'.$activation_url.'">'.__( 'click here', 'woocommerce-product-batch-numbers' ).'</a>' ); ?>
				</p>
			</div>
			<?php
		}

		/**
		 * Loads admin assets.
		 *
		 * @return void
		 */
		public function load_admin_assets() {
			if ( is_admin() && get_current_screen()->id == 'product_page_wpo-batch-numbers' ) {
				wp_enqueue_style( 'wpo-wcpbn-settings-styles', $this->plugin_dir_url . 'assets/css/settings'.$this->assets_suffix.'.css', array(), WPBATCHNUMBERS_VERSION );
				wp_enqueue_style( 'wpo-wcpbn-export-styles', $this->plugin_dir_url . 'assets/css/export'.$this->assets_suffix.'.css', array(), WPBATCHNUMBERS_VERSION );
			}

			if ( is_admin() && in_array( get_current_screen()->id, array( 'edit-product', 'product', 'shop_order', 'edit-shop_order', 'woocommerce_page_wc-orders', 'product_page_wpo-batch-numbers' ) ) ) {
				wp_enqueue_style( 'wpo-wcpbn-styles', $this->plugin_dir_url . 'assets/css/styles'.$this->assets_suffix.'.css', array(), WPBATCHNUMBERS_VERSION );
				wp_enqueue_style( 'wpo-wcpbn-admin-batch-data', $this->plugin_dir_url . 'assets/css/batch-data'.$this->assets_suffix.'.css', array(), WPBATCHNUMBERS_VERSION );

				wp_enqueue_script( 'wpo-wcpbn-scripts', $this->plugin_dir_url . 'assets/js/scripts'.$this->assets_suffix.'.js', array( 'jquery', 'jquery-ui-autocomplete', 'jquery-blockui', 'jquery-ui-tooltip', 'wc-enhanced-select' ), WPBATCHNUMBERS_VERSION, true );
				
				wp_localize_script(
					'wpo-wcpbn-scripts',
					'wpo_wcpbn',
					array(
						'ajaxurl' => admin_url( 'admin-ajax.php' )
					)
				);
			}
			
			if ( is_admin() && get_current_screen()->id == 'product_page_wpo-batch-numbers' ) {
				wp_enqueue_style(
					'select2',
					plugins_url( 'assets/css/select2.css', WC_PLUGIN_FILE ),
					array(),
					WC_VERSION
				);
				
				wp_enqueue_style( 'wpo-wcpbn-admin-batch-tools-styles', $this->plugin_dir_url . 'assets/css/batch-tools'.$this->assets_suffix.'.css', array(), WPBATCHNUMBERS_VERSION );
				
				wp_enqueue_script( 'wc-enhanced-select' );
				
				wp_enqueue_script( 'wpo-wcpbn-admin-batch-tools-scripts', $this->plugin_dir_url . 'assets/js/batch-tools'.$this->assets_suffix.'.js', array( 'jquery', 'jquery-blockui', 'wc-enhanced-select', 'selectWoo' ), WPBATCHNUMBERS_VERSION, true );
				
				wp_localize_script(
					'wpo-wcpbn-admin-batch-tools-scripts',
					'wpo_wcpbn',
					array(
						'ajaxurl' => admin_url( 'admin-ajax.php' ),
						'nonce'   => wp_create_nonce( 'wpo_wcpbn' ),
					)
				);
			}
			
			if ( is_admin() && 'product_page_wpo-batch-numbers' === get_current_screen()->id && isset( $_REQUEST['tab'] ) && 'rest' === $_REQUEST['tab'] ) {
				wp_enqueue_style( 'wpo-wcpbn-prism-styles', $this->plugin_dir_url . 'assets/css/prism'.$this->assets_suffix.'.css', array(), WPBATCHNUMBERS_VERSION );
				wp_enqueue_script( 'wpo-wcpbn-prism-scripts', $this->plugin_dir_url . 'assets/js/prism'.$this->assets_suffix.'.js', array(), WPBATCHNUMBERS_VERSION, true );
			}
			
			if ( is_admin() && get_current_screen()->id == 'edit-product' ) {
				wp_enqueue_script( 'wpo-wcpbn-quick-edit-product', $this->plugin_dir_url . 'assets/js/quick-edit-product'.$this->assets_suffix.'.js', array( 'jquery' ), WPBATCHNUMBERS_VERSION, true );

				wp_localize_script(
					'wpo-wcpbn-quick-edit-product',
					'wpo_wcpbn',
					array(
						'ajaxurl' => admin_url( 'admin-ajax.php' )
					)
				);
			}
			
			if ( is_admin() ) {
				wp_enqueue_script( 'wpo-wcpbn-admin-notices', $this->plugin_dir_url . 'assets/js/admin-notices'.$this->assets_suffix.'.js', array( 'jquery' ), WPBATCHNUMBERS_VERSION, true );

				wp_localize_script(
					'wpo-wcpbn-admin-notices',
					'wpo_wcpbn',
					array(
						'ajaxurl' => admin_url( 'admin-ajax.php' ),
						'nonce'   => wp_create_nonce( 'wpo_wcpbn' ),
					)
				);
			}
		}

		/**
		 * Loads frontend assets.
		 *
		 * @return void
		 */
		public function load_frontend_assets() {
			if ( is_product() || is_cart() ) {
				wp_enqueue_style( 'wpo-wcpbn-frontend-batch-data', $this->plugin_dir_url . 'assets/css/batch-data'.$this->assets_suffix.'.css', array(), WPBATCHNUMBERS_VERSION );
			}
			
			if ( is_product() ) {
				wp_enqueue_script( 'wpo-wcpbn-frontend-product', $this->plugin_dir_url . 'assets/js/product'.$this->assets_suffix.'.js', array('jquery'), WPBATCHNUMBERS_VERSION, true );

				wp_localize_script(
					'wpo-wcpbn-frontend-product',
					'wpo_wcpbn',
					array(
						'ajaxurl'            => admin_url( 'admin-ajax.php' ),
						'nonce'              => wp_create_nonce( 'wpo_wcpbn' ),
						'display_batch_data' => ! empty( WPO_WCPBN()->classes['settings']->options->get_option( 'display_batch_data_in_product_pages' ) ) && 'no' !== WPO_WCPBN()->classes['settings']->options->get_option( 'display_batch_data_in_product_pages' ),
					)
				);
			}
		}

		/**
		 * Load plugin textdomain.
		 *
		 * @return void
		 */
		public function load_textdomain() {
			load_plugin_textdomain( 'woocommerce-product-batch-numbers', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );
		}

		/**
		 * Define constant if not already set
		 * @param  string $name
		 * @param  string|bool $value
		 */
		private function define( $name, $value ) {
			if ( ! defined( $name ) ) {
				define( $name, $value );
			}
		}

		/**
		 * WooCommerce activation check.
		 *
		 * @return bool
		 */
		public function is_woocommerce_activated() {
			$blog_plugins = get_option( 'active_plugins', array() );
			$site_plugins = is_multisite() ? (array) maybe_unserialize( get_site_option( 'active_sitewide_plugins' ) ) : array();

			if ( function_exists( 'WC' ) ) {
				return true;
			} elseif( in_array( 'woocommerce/woocommerce.php', $blog_plugins ) || isset( $site_plugins['woocommerce/woocommerce.php'] ) ) {
				return true;
			} else {
				return false;
			}
		}

		/**
		 * Need WooCommerce notice
		 * 
		 * @return void
		 */
		public function need_woocommerce_notice() {
			/* translators: 1,2: <a> tags */
			$error   = sprintf( __( 'WooCommerce Product Batch Numbers requires %1$sWooCommerce%2$s to be installed & activated!' , 'woocommerce-product-batch-numbers' ), '<a href="http://wordpress.org/extend/plugins/woocommerce/">', '</a>' );
			$message = '<div class="error"><p>' . $error . '</p></div>';
			echo $message;
		}

		/**
		 * Add HPOS compatibility
		 */
		public function custom_order_tables_compatibility() {
			if ( class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
				\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
			}
		}
	}

	/**
	 * Define WPO_WCPBN_TEMPLATES_PATH.
	 */
	if ( ! defined( 'WPO_WCPBN_TEMPLATES_PATH' ) ) {
		define( 'WPO_WCPBN_TEMPLATES_PATH', plugin_dir_path( __FILE__ ) . 'src/Templates' );
	}

}

add_action( 'plugins_loaded', 'WPO_WCPBN' );
function WPO_WCPBN() {
	// Require autoload
	require plugin_dir_path( __FILE__ ) . 'vendor/autoload.php';
	// Intantiate
	return WPO_WCPBN::instance();
}
